#
# Makefile for the micro-mesh firmware
# Supports Msp430 and lpc2148 (arm) architectures.
# (c) 2007 Heiko Will - hwill@inf.fu-berlin.de
#
.PHONY: all clean driver arm

#ARM_CC = /usr/share/gnuarm-3.4.3/bin/arm-elf-gcc
ARM_CC = arm-elf-gcc
ARM_CFLAGS = -O2 -Wall -mcpu=arm7tdmi
ARM_LFLAGS = -mcpu=arm7tdmi -nostartfiles -static -T src/lpc2148/linkerscript.ld -lgcc --warn-common 

ARMSRCS :=  $(wildcard src/*.c)
ARMOBJS :=  $(patsubst src/%.c, bin/arm/%.o, $(ARMSRCS))
ARMDEPS :=  $(patsubst src/%.c, bin/arm/%.d, $(ARMSRCS))

ARM_DRIVER_OBJS = $(wildcard src/lpc2148/bin/*.o)

arm : armdriver $(ARMOBJS)
	$(ARM_CC) $(ARM_LFLAGS) src/lpc2148/src/crt.o $(ARM_DRIVER_OBJS) $(ARMOBJS) -o bin/arm/arm.elf
	objcopy -O ihex bin/arm/arm.elf bin/arm/arm.hex
	size bin/arm/arm.elf

-include $(ARMDEPS)

bin/arm/%.d : src/%.c
	set -e; $(ARM_CC) -MM -I src/driver_api -I src/hal_includes -I src/lpc2148/includes  $< \
	| sed 's/\($*\)\.o[ :]*/bin\/arm\/\1.o bin\/arm\/$*\.d : /g' > $@; \
	[ -s $@ ] || rm -f $@

bin/arm/%.o : src/%.c
	$(ARM_CC) $(ARM_CFLAGS) -I src/driver_api -I src/hal_includes -I src/lpc2148/includes  -c $< -o $@

armdriver:
	make -C src/lpc2148

clean:
	@make clean -C src/lpc2148
	@make clean -C src/msp430
	@rm -f bin/arm/*.o
	@rm -f bin/arm/*.elf
	@rm -f bin/arm/*.d
	