/* Copyright (C) 2005, 2006, 2007, 2008 by Thomas Hillebrandt and Heiko Will

This file is part of the Micro-mesh SensorWeb Firmware.

Micro-Mesh is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3, or (at your option)
any later version.

Micro-Mesh is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Micro-Mesh; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

/* As a special exception, if you include this header file into source
   files compiled by GCC, this header file does not by itself cause
   the resulting executable to be covered by the GNU General Public
   License.  This exception does not however invalidate any other
   reasons why the executable file might be covered by the GNU General
   Public License.  */

#ifndef DRIVER_CC1100_EXTERNALS_H_
#define DRIVER_CC1100_EXTERNALS_H_

#include <stdint.h>

// Some Defines for convinience only
typedef void (*callback_t)(void); 
typedef void (*fp_timer) (void);
typedef int					bool;	/**< #TRUE or #FALSE */
#define	true	1					/**< TRUE */
#define false	0					/**< FALSE */

// this function is called if the cc1100 hangs to signalise error or initiate system reset, etc.
void alarm(void);

// returns a random number between 0 and k-1
int irand(int k);

// toggles a Led
void Led1Toggle(void);

// turns a Led on
void Led2On(void);

// turns a Led off
void Led2Off(void);

// Resets the Microcontroller
void McuReset(void);

// Delets and deactivates a timer with the given id
void delete_timer(int worTimerId);

// waits num*120us (blocking)
void wait_ticks(unsigned int num);

// registers a nonblocking timer. After ticks * 120us the callback is invoked outside an interrupt! Returns the id of the registered timer.
int register_timer(fp_timer fp, unsigned int ticks);

// returns the precise number of Ticks since power on. A tick is 120us. There is no overflow-handling till now!
uint32_t *TICKS();

/* Firmware events*/

typedef struct eventFlags eventFlags;
typedef struct state_info state_info;

struct state_info
{
	unsigned int POWERDOWN:1	;	// Powerdown is active
	unsigned int TIMER_ACTIVE:1	;	// Timer is active
};

// Main firmware event flags
struct eventFlags
{
	union
	{
	struct Bits
	{
		unsigned int UART_EDGE_DETECT	: 1;	// UART start edge detection
		unsigned int TIMER				: 1;	// Timer fired
		unsigned int USER_TIMER			: 1;	// Usertimer fired
	};
		int Int;
	};
};

// -----------------------------------------------------------------------
// Global Vars of the firmware
// -----------------------------------------------------------------------
extern volatile eventFlags mEventFlags;
extern volatile state_info system_state;


#endif /*DRIVER_CC1100_EXTERNALS_H_*/
