/* Copyright (C) 2005, 2006, 2007, 2008 by Thomas Hillebrandt and Heiko Will

This file is part of the Micro-mesh SensorWeb Firmware.

Micro-Mesh is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3, or (at your option)
any later version.

Micro-Mesh is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Micro-Mesh; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include "config.h"
#ifdef INCLUDE_FAT16
#include <stdlib.h>
#include "smallib.h"
#include "dir.h"
#include "fat.h"
 
int dir_get_dir_entry(dir_entry *entry,const char *path);

int dir_opendir(dir_handle *dir,const char *path)
{
	dir_entry entry;
	if (path==NULL || path[0] == '\0') return -1;
	if (strcmp(path,"/") == 0)
	{
		dir->start_cluster = DIR_ROOTDIR;
		dir->akt_cluster = DIR_ROOTDIR;
		dir->entry_position = 0;
		return 0;
	}
	else
	{
		if (dir_get_dir_entry(&entry,path)!=0)
		{
			return -1;
		}
		if (!entry.flags.directory)
		{
			return -1;	
		}
		dir->entry_position = 0;
		dir->start_cluster = entry.start_cluster;
		dir->akt_cluster = entry.start_cluster;
		dir->chain_pos = 0;
	}
	return 0;
}

int dir_get_dir_entry(dir_entry *entry,const char *path)
{
	char buffer[10];
	uint16_t from,to;
	dir_handle dir;
	
	/*Die Länge des gesamten Pfadnamens*/
	dir.entry_position=0;
	dir.start_cluster=DIR_ROOTDIR;

	from=0;
	if (path[0]=='/') from=1;
	to=from;
	while (1)
	{
		to++;
		if (*(path+to)=='/'||*(path+to+1)=='\0')
		{
			if (*(path+to)=='/')to--;
			memcpy(buffer,path+from,to-from+1);
			buffer[to-from+1]=0;
			while (1)
			{
			  if (dir_readdir(&dir,entry)!=0) 
			  {
			  	return -1;
			  }	
			  else
			  {
			  	if (strcmp(entry->filename,buffer)==0)
			  	{
			  		if (*(path+to+1)=='\0') 
			  		{
			  			return 0;
					}
					else
					{
						dir.start_cluster=entry->start_cluster;
						dir.entry_position=0;
						dir.akt_cluster=entry->start_cluster;
						dir.chain_pos=0;
						from=to+2;
			  			to=from;
			  			break;
					}
			  	}
			  }
			}
		}
		
		if (to-from==9) 
		{
			return -1;	
		}
	}
}

raw_dir_entry *dir_readdir_raw(dir_handle *handle)
{
		static raw_dir_entry *res;
		uint8_t *buffer;
		
		/* Search in rootdir*/
		if (handle->start_cluster == DIR_ROOTDIR)
		{
			buffer = fat_read_cb(vi.root_dir_sector + ((handle->entry_position * 32) / 512)) + 
					 ((handle->entry_position) % 16) * 32;
			if (buffer==NULL) return NULL;
			return (raw_dir_entry*)(buffer);
		}
		
	 	/*Search not in rootdir*/	 	
		uint32_t sector = ((handle->entry_position * 32) / 512);
		uint16_t cluster_count = sector / vi.sec_per_cluster;
		sector = sector % vi.sec_per_cluster;
		uint16_t i = 0;
		uint16_t cluster = handle->akt_cluster;
		
		if (cluster_count != handle->chain_pos)
		{
			/* Speed up for forward cluster change, this is the common
			 * branch, because forward seeking is using it. So no other speedups 
			 * meke sense compared with there codesize disadvantages.*/
			if (cluster_count == handle->chain_pos+1)
			{
				cluster = fat_get_next_cluster( cluster);
			}
			/*Search clusterchain from the beginning*/
			else
			{
				cluster = handle->start_cluster;
				for (i=0;i<cluster_count;i++)
				{
					cluster = fat_get_next_cluster( cluster);
				}
			}
			handle->chain_pos=cluster_count;
		}
		
		if (cluster==FAT_END_OF_CHAIN) 
			return NULL;
		handle->akt_cluster=cluster;
						
		buffer = fat_read_cb(vi.data_sector+(cluster-2)*vi.sec_per_cluster+sector);
	
		res = (raw_dir_entry*)(buffer+(handle->entry_position*32)%512);

		return res;	
}

int dir_readdir(dir_handle *handle,dir_entry *res)
{
	int b=0;
	raw_dir_entry *entry;
		
	/*Prüfen ob es ein 8.3 Eintrag ist
	//Das ist der Fall wenn es sich nicht um einen Eintrag für lange Dateinamen
	//oder um einen als gelöscht markierten Eintrag handelt.
	*/
	while (1)
	{
		entry = dir_readdir_raw(handle);
		if (entry == NULL) return -1;
		if (entry->filename[0] == '\0') /*Kein weiterer Eintrag wenn erstes Zeichen des Namens 0 ist*/
		{
			return DIR_END_OF_DIR;
		}
		handle->entry_position++;
		
   		if ((entry->flags.asbyte != DIR_ATTR_VFAT_NAME) && (entry->filename[0] != (char) DIR_ENTRY_IS_FREE)) 
		{
				/*Speichern des Verzeichnis Eintrages*/ 
				res->flags.asbyte = entry->flags.asbyte;
				b=0;
				while (entry->filename[b] != DIR_NAME_TERMINATOR && b<8)
				{
					res->filename[b]=entry->filename[b];
					b++;
				}
				res->filename[b]='\0';
				b=0;
				while (entry->extension[b] != DIR_NAME_TERMINATOR && b<3)
				{
					res->extension[b]=entry->extension[b];
					b++;
				}
				res->extension[b]='\0';
					
				/*Speichern der Filegröße*/
				/*Hilfskonstrukt für msp430 der nur auf gerade wörter zugreifen kann*/
				uint8_t *hlp = (uint8*) &entry->filesize;
				res->filesize = (((uint32)(*(hlp+3)))<<24) + (((uint32)(*(hlp+2))<<16)) + (((uint32)(*(hlp+1))<<8)) +(uint32)*(hlp) ;
				/*Speichern des Clusters des Verzeichniseintrages*/
				/*Hilfskonstrukt für msp430 der nur auf gerade wörter zugreifen kann*/
				hlp = (uint8*) &entry->cluster;
				res->start_cluster = (((uint16_t)(*(hlp+1)))<<8) + (uint16_t)(*hlp);
				return 0;
		}
	}
}

int dir_write_entry(dir_handle *handle, raw_dir_entry *newentry)
{
	uint8* ret;
	raw_dir_entry *oldentry;
	uint32_t sector;
	//sector laden
	oldentry = dir_readdir_raw(handle);
	//neuen entry kopieren
	sl_memcpy(oldentry,newentry,32);
	// relativer sektor zum cluster
	sector = ((handle->entry_position * 32) / 512) % vi.sec_per_cluster;
	// sector schreiben
	if (handle->start_cluster == DIR_ROOTDIR)
		ret = fat_write_cb(vi.root_dir_sector + ((handle->entry_position * 32) / 512));
	else
		ret = fat_write_cb(vi.data_sector+(handle->akt_cluster-2)*vi.sec_per_cluster+sector);	
	if (ret==NULL) return -1;
	return 0;
}

#endif





