/* Copyright (C) 2005, 2006, 2007, 2008 by Thomas Hillebrandt and Heiko Will

This file is part of the Micro-mesh SensorWeb Firmware.

Micro-Mesh is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3, or (at your option)
any later version.

Micro-Mesh is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Micro-Mesh; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include "config.h"

#ifdef INCLUDE_DISPLAY

#include "display.h"
#include "timer.h"
#include "f9x14.h"
#include "driver_display.h"

#define SCREEN_COLOR            BLUE  // sollte ge�ndert werden f�r andere Hintergrundfarbe
#define COLOR_TABLE_BITS        2      // 2^2 = 4 Farben in Farbtabelle, wichtig f�r Fonts
#define FONT_HEADER_SIZE        7      // Header gr��e eines Fonts

#define RGB(r,g,b) ((unsigned int)((r & 0x00F8) << 8) | ((g & 0x00FC) << 3) | ((b & 0x00F8) >> 3))

#define NONE_C                  RGB(0x00, 0x20, 0x00)  // == tansparent, ein gr�n Ton weniger !!
#define BLACK                   RGB(0x00, 0x00, 0x00)  // f�r 12 Bit Farben R,G,B  4,4,4 Bits
#define WHITE                   RGB(0xFF, 0xFF, 0xFF)
#define RED                     RGB(0xFF, 0x00, 0x00)
#define GREEN                   RGB(0x00, 0xFF, 0x00)
#define BLUE                    RGB(0x00, 0x00, 0xFF)
#define YELLOW                  RGB(0xFF, 0xFF, 0x00)
#define MAGENTA                 RGB(0xFF, 0x00, 0xFF)
#define CYAN                    RGB(0x00, 0xFF, 0xFF)
#define GRAY                    RGB(0x80, 0x80, 0x40)
#define SILVER                  RGB(0xA0, 0xA0, 0x80)
#define GOLD                    RGB(0xA0, 0xA0, 0x40)


// Epson S1D15G10 Command Set

#define SOFT_RESET             0x01	
#define DISPLAY_STATUS         0x09
#define SLEEP_IN               0x10
#define SLEEP_OUT              0x11
#define PARTIAL_MODE_ON        0x12
#define NORMAL_MODE_ON         0x13
#define INVERSION_OFF          0x20
#define INVERSION_ON           0x21
#define PIXELS_OFF             0x22 // all pixels off
#define PIXELS_ON              0x23 // all pixels on
#define SET_CONTRAST           0x25 // write contrast
									// CONTRAST_VALE    0xXX
#define DISPLAY_OFF            0x28
#define DISPLAY_ON             0x29
#define SET_X_ADDR             0x2A // column address set
	// START_X		 0xXX // ATENTION! when X,Y address are changed BOTH must be changed
	// STOP_X		 0xXX
#define SET_Y_ADDR             0x2B // page address set
	// START_Y					         0xXX
	// STOP_Y					           0xXX
#define MEM_WRITE              0x2C
	// x times pixel data		     0xXX
#define COLOR_SET              0x2D
	// 8 times red palette		   0xXX
	// 8 times green palette	   0xXX
	// 4 times blue palette		   0xXX
#define PARTIAL_AREA           0x30
	// START_Y					         0xXX
	// STOP_Y					           0xXX
#define SCROLL_DEFINITON       0x33 // vertical scroll definition
	// TOP_FIXED_LINES	         0xXX
	// SCROLL_LINES			         0xXX
	// BOTTOM_FIXED_LINES		     0xXX
#define MEM_CONTROL            0x36 // memory access control
	// MEM_FLAGS                 0xXX
#define MEM_90               0    // virtual flag not for PCF8833
#define MEM_RGB              3    // color order
#define MEM_LAO              4    // line access order
#define MEM_VW               5    // vertical write
#define MEM_MX               6    // mirror X
#define MEM_MY               7    // mirror Y  
#define SET_SCROLL             0x37 // vertical scrolling start address
	// ROW_Y					           0xxx
#define IDLE_OFF               0x38
#define IDLE_ON                0x39
#define COLOR_INTERFACE        0x3A // interface pixel format
	// pixelformat               0xXX
#define COLOR_8_BIT          0x02
#define COLOR_12_BIT         0x03
#define COLOR_16_BIT         0x05
#define NO_OPERATION           0xAA // NO operation2
#define GRAY_SCALE_POS_SET0    0xB3
#define GRAY_SCALE_POS_SET1    0xB4
#define DISPLAY_CONTROL        0xB6
#define DRIVER_POS             0xBB // Driver position

#define glcdSetColors(fgColor, bkColor) {glcdSetFgColor(fgColor); glcdSetBkColor(bkColor);}
#define glcdPutPixel(color) {glcdDisplayData((0xFF00 & color) >> 8); glcdDisplayData(color);}
#define glcdClearScreen() {glcdFillRect(0, 0, glcd_Screen.SCREEN_WIDTH -1, glcd_Screen.SCREEN_HEIGHT -1, SCREEN_COLOR);}
#define glcdSetScreen(r,x1,y1,x2,y2,x3,y3) {r.SCREEN_WIDTH = x1; r.SCREEN_HEIGHT = y1; r.SCREEN_LEFT = x2; r.SCREEN_TOP = y2; r.SCREEN_RIGHT = x3; r.SCREEN_BOTTOM = y3;}
#define glcdSetRect(r,x1,y1,x2,y2) {r.X1 = x1; r.Y1 = y1; r.X2 = x2; r.Y2 = y2;}
#define glcdSwapCoord(x, y) {glcdCoord_t t = x; x = y; y = t;}
#define glcdCheckCoord(x, y) {if (x > y) {glcdSwapCoord(x, y)}}
#define glcdMoveTo(x, y) {glcd_Cursor.X = x; glcd_Cursor.Y = y;}

#define glcdBkColor glcd_Colors[0]
#define glcdFgColor glcd_Colors[1]
#define glcdFrColor glcd_Colors[2]
#define glcdShColor glcd_Colors[3]
#define glcdSetColor(index, color) {glcd_Colors[index] = color;}
#define glcdSetBkColor(color) {glcdSetColor(0, color);}
#define glcdSetFgColor(color) {glcdSetColor(1, color);}
#define glcdSetFrColor(color) {glcdSetColor(2, color);}
#define glcdSetShColor(color) {glcdSetColor(3, color);}



glcdRect_t      glcd_Clip;          // Clipping Koordinaten, alles was ausserhalb ist wird nicht dargestellt
glcdFlags_t     glcd_Flags;
glcdScreen_t    glcd_Screen;        // aktuelle Fenstergr��e
glcdInit_t      glcd_Init;          // aktuelle Initialwerte

glcdColor_t     glcd_Colors[1 << COLOR_TABLE_BITS]; // Farbtabelle, Colors[0] = Hintergrund = F�llfarbe, Color[1] = Vordergrund = Linienfarbe
glcdRect_t      glcd_Window;          // aktuelles Fenster f�r Textausgaben
glcdFontFunc_t  glcd_FontRead;        // User definierte Readcallback f�r die Fontdaten
glcdFontData_t  glcd_FontData;        // die eigentlichen Fontdaten beginnen hier, abh�ngig von der Readcallback
glcdFontData_t  glcd_PictureData;     // die eigentlichen Bilddaten beginnen hier
glcdPoint_t     glcd_Cursor;          // aktuelle Position f�r Schriften


unsigned char   glcd_FontFirstChar;   // erstes Zeichen im Font
unsigned char   glcd_FontLastChar;    // letztes Zeichen im Font
unsigned char   glcd_FontWidth;       // fixed Zeichenbreite des Fonts
unsigned char   glcd_FontHeight;      // Zeichenh�he
unsigned char   glcd_FontBitsPixel;   // Farbtiefe des Fonts
unsigned char   glcd_DisplayTyp;      // Displaytyp (Epson oder Phillips)

static const char*  BackBuffer[9];
static unsigned char BackBufferPos;
static unsigned char BackBufferFill;

// this can be the basis for more complex graphics
void LCD_put_pixel(unsigned char color, unsigned char x, unsigned char y);
void glcdDoFillRect(glcdCoord_t x1, glcdCoord_t y1, glcdCoord_t x2, glcdCoord_t y2, const glcdColor_t color);
void glcdSetAddr(glcdCoord_t x1, glcdCoord_t y1, glcdCoord_t x2, glcdCoord_t y2);
inline unsigned char glcdFontRead(const glcdFontData_t index);
void glcdSelectFont(const void * data, glcdFontFunc_t callback);
void glcdNewLine(void);
uint16_t glcdPrint_P(const char *addr);

static unsigned char   glcd_MemCtrl;

void initDisplay (void) {
	LoadDisplayDriver();
	wait_ticks(1);
	
	glcdSetScreen(glcd_Screen, 133,133,0,0,131,131);
  	glcdDisplayCommand(SOFT_RESET);
  	wait_ticks(1);
    glcdDisplayCommand(COLOR_INTERFACE);
    glcdDisplayData(COLOR_16_BIT);
    glcdDisplayCommand(MEM_CONTROL);
    // use for wrong orientation of coordinate y
    glcd_MemCtrl = (0 << MEM_MX) | (1 << MEM_MY) | (0 << MEM_RGB) | (1 << MEM_VW) | (0 << MEM_LAO) | (1 << MEM_90);
    glcdDisplayData(glcd_MemCtrl);

    glcdDisplayCommand(INVERSION_OFF);
 
    glcdSetRect(glcd_Clip, glcd_Screen.SCREEN_LEFT, glcd_Screen.SCREEN_TOP, glcd_Screen.SCREEN_RIGHT, glcd_Screen.SCREEN_BOTTOM);
    glcdSetRect(glcd_Window, glcd_Screen.SCREEN_LEFT+3, glcd_Screen.SCREEN_TOP+1, glcd_Screen.SCREEN_RIGHT, glcd_Screen.SCREEN_BOTTOM);
    glcd_Cursor.X  = glcd_Window.X1;
    glcd_Cursor.Y  = glcd_Window.Y1;
      
    glcdSetBkColor(SCREEN_COLOR);     // Hintergrund
    glcdSetFgColor(WHITE);            // Vordergrund
    glcdSetFrColor(GRAY);             // Rahmen
    glcdSetShColor(SILVER);           // Schatten

 
    glcdClearScreen();
    
    glcdSelectFont(f9x14, glcdFontRead);
 
    glcdDisplayCommand(SLEEP_OUT);

    glcdDisplayCommand(DISPLAY_ON);
	glcd_Flags.FixedFont=1;
	glcd_Flags.AutoLineFeed=0;
	
    glcdDisplayCommand(SET_CONTRAST);
    glcdDisplayData(55);   
}

inline unsigned char glcdFontRead(const glcdFontData_t index)
  {
    return *index;
  }

void glcdSelectFont(const void * data, glcdFontFunc_t callback)
  {
  glcd_FontData      = (glcdFontData_t)data;
  glcd_FontRead      = callback;
  glcd_FontWidth     = glcd_FontRead(glcd_FontData + 2);
  glcd_FontHeight    = glcd_FontRead(glcd_FontData + 3);
  glcd_FontBitsPixel = glcd_FontRead(glcd_FontData + 4);
  glcd_FontFirstChar = glcd_FontRead(glcd_FontData + 5);
  glcd_FontLastChar  = glcd_FontRead(glcd_FontData + 6);
  }

void glcdNewLine(void)
  {
  glcd_Cursor.X  = glcd_Window.X1;
  glcd_Cursor.Y += glcd_FontHeight;
  if (glcd_Cursor.Y + glcd_FontHeight > glcd_Window.Y2)
    {
   	glcd_Cursor.Y = glcd_Window.Y1;
    }
  }

unsigned char glcdDrawChar(unsigned char c)
  {
  if (c == '\n')
    {
    glcdNewLine();
    return(0);
    }
  // ist das Zeichen im Font definiert und auch ein Font selektiert ??
  if ((c < glcd_FontFirstChar) || (c > glcd_FontLastChar) || (glcd_FontData == 0))
    {
    return(0);
    }
  // ist das Zeichen im Font deklariert aber nicht definiert ??
  unsigned char charwidth = glcd_FontRead(glcd_FontData + FONT_HEADER_SIZE + c - glcd_FontFirstChar);
  if (charwidth == 0) {return(0);}

  // alle Zeichen kleiner #128 haben rechts eine implizite Leerspalte
  unsigned char width = charwidth;
  if (c < 128) {width++;}
  // wenn wir den Zeichensatz mit fester Breite zeichnen sollen dann setze die Width entsprechend
  if ((glcd_Flags.FixedFont == 1) && (width <= glcd_FontWidth))
    {
    width = glcd_FontWidth;
    }
    
  // passt das Zeichen noch in das Zeichen-Fenster ?? falls nein eventuell ein Linefeed einf�gen
  if ((glcd_Flags.AutoLineFeed == 1) && (glcd_Cursor.X + width >= glcd_Window.X2))
    {
    glcdNewLine();
    }
  // ist das Zeichen �berhaupt noch ansatzweise sichtbar ?
  if ((glcd_Cursor.X > glcd_Clip.X2) || (glcd_Cursor.Y > glcd_Clip.Y2))
    {
    return(0);
    }

  glcdCoord_t x = glcd_Cursor.X;
  glcdCoord_t y = glcd_Cursor.Y;
  glcd_Cursor.X += width;

  // ist das Zeichen �berhaupt noch ansatzweise sichtbar ?
  if ((x > glcd_Clip.X2) || (y + glcd_FontHeight < glcd_Clip.Y1))
    {
    return(0);
    }

  unsigned char i = 0;

  // falls FixedFont m�ssen die Zeichen innerhalb glcd_FontWidth zentriert werden, aber nur
  // wenn das Zeichen schmaler als glcd_FontWidth ist. Wir f�llen dann auch gleich den rechten
  // und linken Rand des Zeichens, ebenfalls nur wenn die Hintergrundfarbe nicht transparent ist.
  if ((glcd_Flags.FixedFont == 1) && (width > charwidth))
    {
    i = (width - charwidth) / 2;
    if (glcdBkColor != NONE_C)
      {
      glcdDoFillRect(x, y, x + i -1, y + glcd_FontHeight -1, glcdBkColor);
      }
    x += i;
    }
  i = width - charwidth - i;
  if ((i > 0) & (glcdBkColor != NONE_C))
    {
    glcdDoFillRect(x + charwidth, y, x + charwidth + i -1, y + glcd_FontHeight -1, glcdBkColor);
    }

  // m�ssen die eigentlichen Zeichendaten gezeichnet werden ??
  if ((x > glcd_Clip.X2) || (x + charwidth < glcd_Clip.X1))
    {
    return(width);
    }

  glcdFontData_t data;                                   // *Font Daten
  unsigned char bitspixel = glcd_FontBitsPixel & 0x7F;   // Bits pro Pixel
  unsigned char bitsmask = 0xFF >> (8 - bitspixel);      // Bitmaske f�r Farbe
  uint16_t bits;                                         // enth�lt die aktuelle Bits des Zeichens
  unsigned char bitscount;                               // aktuelle Anzahl an Bits in Bits
  unsigned char table[4];                                // RLE L�ngen Tabelle

  // ALLE Lesevorg�nge in den Font-Speicher sind so aufgebaut das sie ganze Bl�cke von
  // Bytes sequentiell erledigen. Dies ist wichtig und sehr gut optimiert auf zB. I2C EEPROMS.

  if (glcd_FontBitsPixel & 0x80)
    {
    // komprimierte Fonts

    // Hole das BytePadding die RLE L�ngentabelle und berechne den Byteindex zu den Daten.
    // Dieser Lesevorgang hat einen zus�tzliche Overhead von 4 bytes bei komprimierten Fonts.
    // Allerdings wird dieser Overhead bei weitem in der eigentlichen Fontzeichendarstellungsschleife
    // wieder wett gemacht !
    // Bei komprimierten Fonts kann kein linkes Clipping vorgenommen werden :(
    data = glcd_FontData + FONT_HEADER_SIZE + glcd_FontLastChar - glcd_FontFirstChar +1;
    unsigned char padding = glcd_FontRead(data++);
    table[0] = 1;
    table[1] = glcd_FontRead(data++);
    table[2] = glcd_FontRead(data++);
    table[3] = glcd_FontRead(data++);
    uint32_t index = 0;
    for (i = glcd_FontFirstChar; i < c; i++)
      {
      index += glcd_FontRead(data++);
      }
    index *= padding;
    data = glcd_FontData + FONT_HEADER_SIZE + (glcd_FontLastChar - glcd_FontFirstChar +3) * 2 + index;
    bits = glcd_FontRead(data++);
    bitscount = 8;
    }
  else
    {
    // unkomprimierte Fonts

    // berechne nun den Bitindex zu den Bitdaten im Font
    data = glcd_FontData + FONT_HEADER_SIZE;
    uint32_t index = 0;
    for (i = glcd_FontFirstChar; i < c; i++)
      {
      index += glcd_FontRead(data++);
      }
    // linkes Clipping ber�cksichtigen, nur bei unkomprimierten Fonts m�glich
    if (x < glcd_Clip.X1)
      {
      glcdCoord_t t = glcd_Clip.X1 - x;
      index += t;
      charwidth -= t;
      x = glcd_Clip.X1;
      }
    index *= glcd_FontHeight * bitspixel;

    // positioniere data auf's erste Byte mit Fontdaten zum Zeichen und hole die ersten Bits
    bitscount = index % 8;
    index /= 8;
    data = glcd_FontData + FONT_HEADER_SIZE + glcd_FontLastChar - glcd_FontFirstChar +1 + index;
    bits = glcd_FontRead(data++) >> bitscount;
    bitscount = 8 - bitscount;
    }


  unsigned char pixelcount = 0;                           // Anzahl der zu setzenden Pixel
  glcdColor_t pixelcolor = 0;                             // zu setzende Pixelfarbe
  unsigned char clipinvalid = 1;                          // G�ltigkeit der Adresszeiger im PCF8833
  glcdCoord_t clipY = 0;                                  // letzte gesetzte Y-Adresse im PCF8833
  glcdCoord_t stopY = glcd_Cursor.Y + glcd_FontHeight -1; // unterste Pixelzeile des Zeichens
  glcdCoord_t stopX = x + charwidth -1;

  // rechtes Clipping ber�cksichtigen
  if (stopX > glcd_Clip.X2)
    {
    stopX = glcd_Clip.X2;
    }

  for (; x <= stopX; x++)
    {
    if (clipY |= glcd_Cursor.Y)
      {
      clipinvalid = 1;
      }
    for (y = glcd_Cursor.Y; y <= stopY; y++)
      {
      if (pixelcount == 0)
        {
        if (bitscount <= 8)
          {
          bits |= glcd_FontRead(data++) << bitscount;
          bitscount += 8;
          }
        if (glcd_FontBitsPixel & 0x80)
          {
          pixelcount = table[bits & 3];
          bits >>= 2;
          bitscount -= 2;
          }
        else
          {
          pixelcount++;
          }
        if (bitspixel <= COLOR_TABLE_BITS)
          {
          pixelcolor = glcd_Colors[bits & bitsmask];
          }
        else
          {
          pixelcolor = bits & bitsmask;
          }
        bits >>= bitspixel;
        bitscount -= bitspixel;
        }
      pixelcount--;
      if ((pixelcolor != NONE_C) & (y >= glcd_Clip.Y1) & (y <= glcd_Clip.Y2) & (x >= glcd_Clip.X1))
        {
        if (clipinvalid)
          {
          glcdSetAddr(x, y, glcd_Cursor.X, stopY);
          clipY = y;
          clipinvalid = 0;
          }
        glcdPutPixel(pixelcolor);
        }
      else
        {
        clipinvalid = 1;
        }
      }
    }


  return(width);
}
//******************************************************************************
unsigned char glcdCharWidth(unsigned char c)
  {
  if ((c < glcd_FontFirstChar) | (c > glcd_FontLastChar) | (glcd_FontData == 0))
    {
    return(0); // character not defined
    }
  else
    {
    unsigned char i = glcd_FontRead(glcd_FontData + FONT_HEADER_SIZE + c - glcd_FontFirstChar);
    if (i > 0)
      {
      if (c < 128){i++;}
      if ((glcd_Flags.FixedFont) & (i < glcd_FontWidth))
        {
        i = glcd_FontWidth;
        }
      }
    return(i);
    }
  }
//******************************************************************************
uint16_t glcdPrint(const char *addr)
  {
  unsigned char c;
  uint16_t i = 0;

  while ((c = *addr++))
    {
    i += glcdDrawChar(c);
    }
  return(i);
  }

void glcdWrite(const char *line)
{
	glcd_Cursor.Y=glcd_Window.Y1;
	glcd_Cursor.X=glcd_Window.X1;
	BackBuffer[BackBufferPos]=line;
	BackBufferPos++;
	if (BackBufferFill<9)BackBufferFill++;
	BackBufferPos=BackBufferPos%9;
	int c=0,i=0;
	if (BackBufferFill<9)
	{
		for (i=0;i<BackBufferFill;i++)
			glcdPrint(BackBuffer[i]);
	}
	else
	{
		i=(BackBufferPos+1)%9;
		while (c<9)
		{
			glcdPrint(BackBuffer[i]);
			i=(i+1)%9;
			c++;
		}
	}
}


//******************************************************************************
uint16_t glcdPrint_P(const char *addr)
  {
  unsigned char c;
  uint16_t i = 0;
  glcdSetBkColor(NONE_C);

  while((c =*addr++))
    {
    i += glcdDrawChar(c);
    }
  return(i);
  }
//******************************************************************************
uint16_t glcdCharsWidth(const char *addr)
  {
  unsigned char c;
  uint16_t i = 0;

  while ((c = *addr++))
    {
    i += glcdCharWidth(c);
    }
  return(i);
  }
//******************************************************************************
uint16_t glcdCharsWidth_P(const char *addr)
  {
  unsigned char c;
  uint16_t i = 0;

  while((c =*addr++))
    {
    i += glcdCharWidth(c);
    }
  return(i);
  }


void glcdSetAddr(glcdCoord_t x1, glcdCoord_t y1, glcdCoord_t x2, glcdCoord_t y2)
  {
  if(glcd_MemCtrl & (1 << MEM_90))
    {
    glcdSwapCoord(x1, y1);
    glcdSwapCoord(x2, y2);
    }
  glcdDisplayCommand(SET_X_ADDR);
  glcdDisplayData(x1);
  glcdDisplayData(x2);
  glcdDisplayCommand(SET_Y_ADDR);
  glcdDisplayData(y1);
  glcdDisplayData(y2);
  glcdDisplayCommand(MEM_WRITE);
  }

void glcdDoFillRect(glcdCoord_t x1, glcdCoord_t y1, glcdCoord_t x2, glcdCoord_t y2, const glcdColor_t color)
  {
  glcdCoord_t x, y;

  if ((x1 > glcd_Clip.X2) | (x2 < glcd_Clip.X1) | (y1 > glcd_Clip.Y2) | (y2 < glcd_Clip.Y1)) {return;}
  if (x1 < glcd_Clip.X1) {x1 = glcd_Clip.X1;}
  if (y1 < glcd_Clip.Y1) {y1 = glcd_Clip.Y1;}
  if (x2 > glcd_Clip.X2) {x2 = glcd_Clip.X2;}
  if (y2 > glcd_Clip.Y2) {y2 = glcd_Clip.Y2;}
  if ((x1 <= x2) & (y1 <= y2))
    {
    glcdSetAddr(x1, y1, x2, y2);
    for (y = y1; y <= y2; y++)
      {
      for (x = x1; x <= x2; x++)
        {
          glcdPutPixel(color);
        }
      }
    }
  }

void glcdFillRect(glcdCoord_t x1, glcdCoord_t y1, glcdCoord_t x2, glcdCoord_t y2, const glcdColor_t color)
  {
  glcdCheckCoord(x1, x2);
  glcdCheckCoord(y1, y2);
  glcdDoFillRect(x1, y1, x2, y2, color);
  }

void glcdSetRectWindow(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2) {
  glcdSetRect(glcd_Window, x1, y1, x2, y2);
}

void DisplaySleep(void)
{
	DisplayBacklightOff();
	glcdDisplayCommand(DISPLAY_OFF);
	
}

void DisplayWakeup(void)
{
	glcdDisplayCommand(DISPLAY_ON);
	DisplayBacklightOn();
}
#endif
