/* Copyright (C) 2005, 2006, 2007, 2008 by Thomas Hillebrandt and Heiko Will

This file is part of the Micro-mesh SensorWeb Firmware.

Micro-Mesh is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3, or (at your option)
any later version.

Micro-Mesh is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Micro-Mesh; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

/* As a special exception, if you include this header file into source
   files compiled by GCC, this header file does not by itself cause
   the resulting executable to be covered by the GNU General Public
   License.  This exception does not however invalidate any other
   reasons why the executable file might be covered by the GNU General
   Public License.  */

#ifndef DISPLAY_H_
#define DISPLAY_H_
#include "config.h"
#include "type.h"

#ifdef INCLUDE_DISPLAY
	#define glcd_write(i) glcdWrite(i)
#else
	#define glcd_write(i) 
#endif 


#ifdef INCLUDE_DISPLAY
 
 typedef struct
  {
  uint8_t SCREEN_WIDTH;
  uint8_t SCREEN_HEIGHT;
  uint8_t SCREEN_LEFT;
  uint8_t SCREEN_TOP;
  uint8_t SCREEN_RIGHT;
  uint8_t SCREEN_BOTTOM;
  } glcdScreen_t;
  
  typedef unsigned char glcdCoord_t;                                   // Koordinaten
  typedef uint16_t glcdCoord2_t;     
  typedef unsigned char *glcdFontData_t;                               // zeiger auf Fontdaten
  typedef unsigned char (*glcdFontFunc_t)(const glcdFontData_t index); // Fontdaten-Lese-Callback, somit k�nnen
  
  
  typedef struct
    {
    glcdCoord_t X;
    glcdCoord_t Y;
    } glcdPoint_t;

  typedef struct
    {
    glcdCoord_t X1;
    glcdCoord_t Y1;
    glcdCoord_t X2;
    glcdCoord_t Y2;
    } glcdRect_t;
    
    typedef struct
    {
    unsigned char AutoLineFeed:1,     // Fontroutine nutzt einen impliziten Zeilenumbruch innerhalb von glcd_Window
    FixedFont:1;                      // Fontroutine soll Font mit fester Zeichenbreite darstellen, ansonsten proportional
    } glcdFlags_t;
    
    typedef unsigned int glcdColor_t;    
    
    typedef struct
      {
      uint8_t SCREEN_VOL;
      uint8_t SCREEN_ORIENTATION;
      uint8_t SCREEN_COLOR_BITS;
      uint8_t SCREEN_INVERSION;
      } glcdInit_t;

void initDisplay (void);
void glcdFillRect(glcdCoord_t x1, glcdCoord_t y1, glcdCoord_t x2, glcdCoord_t y2, const glcdColor_t color);
uint16_t glcdPrint(const char *addr);
void glcdWrite(const char *line);
void DisplaySleep(void);
void DisplayWakeup(void);

#endif /*INCLUDE_DISPLAY*/
#endif /*DISPLAY_H_*/
