/* Copyright (C) 2005, 2006, 2007, 2008 by Thomas Hillebrandt and Heiko Will

This file is part of the Micro-mesh SensorWeb Firmware.

Micro-Mesh is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3, or (at your option)
any later version.

Micro-Mesh is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Micro-Mesh; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

/* As a special exception, if you include this header file into source
   files compiled by GCC, this header file does not by itself cause
   the resulting executable to be covered by the GNU General Public
   License.  This exception does not however invalidate any other
   reasons why the executable file might be covered by the GNU General
   Public License.  */

#include "config.h"
#ifdef INCLUDE_FAT16
#ifndef FAT_H_
#define FAT_H_
#include "type.h"

#define FAT_END_OF_CHAIN (0xffff)
#define FAT_INCOMPATIBLE_SECTOR_SIZE (int) -2

typedef struct __attribute__ ((packed))  _volume_record{
	uint8_t jmp[3];
	char oem_name[8];
	uint16_t bytes_per_sector;
	uint8_t sectors_per_cluster;
	uint16_t reserved_sectors;
	uint8_t fat_copies;
	uint16_t root_dir_entries;
	uint16_t deprecated1;
	uint8_t media_type;
	uint16_t sectors_per_fat;
	uint16_t sectors_per_track;
	uint16_t number_of_heads;
	uint32_t reserved_begin_count;
	uint32_t sector_count;
	uint16_t drive_id;
	uint8_t  deprecated2;
	uint32_t serial_number;
	char volume_name[11];
	char fat_name[8];
	uint8_t executable_code[448];
	uint16_t signature;
} volume_boot_record ;

typedef uint8* (*fat_io_callback) (uint32);
typedef int (*fat_flush_callback) (void);

typedef struct _volume_info {
	char   volume_name[9];
	uint8_t  sec_per_cluster;
	uint16_t vbr_sector; /*Offset in Sectors*/
	uint16_t fat_sector;
	uint16_t root_dir_sector;
	uint16_t data_sector;
	uint16_t fat_size;
	uint16_t bytes_per_Sector; /*2 bytes*/
} volume_info;

extern volume_info vi;
extern fat_io_callback fat_read_cb;
extern fat_io_callback fat_write_cb;
extern fat_flush_callback fat_flush_cb;

int fat_init_media(fat_io_callback rcb,fat_io_callback wcb,fat_flush_callback fcb);

// Gets the next Cluster in a chain of clusters.
int fat_get_next_cluster(uint16_t cluster);

// Gets the first unused cluster of the FAT-Table
uint16_t fat_get_first_free_cluster(void);

// Adds a cluster to a chain of clusters
uint16_t fat_add_cluster(uint16_t cluster);

// Unlinks all following clusters of a chain of clusters
int fat_unlink_chain(uint16_t cluster);

// Writes an entry of the FAT
int fat_write_cluster(uint16_t cluster,uint16_t link);

#endif /*FAT_H_*/
#endif
