/* Copyright (C) 2005, 2006, 2007, 2008 by Thomas Hillebrandt and Heiko Will

This file is part of the Micro-mesh SensorWeb Firmware.

Micro-Mesh is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3, or (at your option)
any later version.

Micro-Mesh is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Micro-Mesh; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include "config.h"
#ifdef INCLUDE_FAT16
#include <stdlib.h>
#include <stdio.h>
#include "file.h"
#include "fat.h"
#include "smallib.h"
#include "dir.h"
 
int file_createfile(file_handle *file,dir_handle *handle,const char *path);

int file_openfile(file_handle *file,const char *path,const char mode)
{
	char tmpname[13];
	dir_entry entry;
	dir_handle dir;
	int search_res=0;
	int pathlen;
	int a;
 	
	if (sl_strcmp(path,"/")==0)
	{
		return -1;
	}
		
	pathlen=sl_strlen(path);

 
	/* Filenamen und Path trennen*/
	for (a=pathlen-1;a>=0;a--)
		if (path[a]=='/') break;
	char fpath[a+2];
	sl_memcpy(fpath,path,a+1);
	fpath[a+1]=0;
	char filename[pathlen-a];
	sl_memcpy(filename,path+a+1,pathlen-a);

	/* Pfad suchen*/
	/* Pfad existiert nicht*/
	if (dir_opendir(&dir,fpath)!=0) 
	{
		return -2;
	}
	
	/* und nach Filename suchen, ggf anlegen!*/
	while((search_res=dir_readdir(&dir,&entry))!= DIR_END_OF_DIR)
	{
		sl_strcpy(tmpname,entry.filename);
		sl_strcat(tmpname,".");
		sl_strcat(tmpname,entry.extension);
		if (sl_strcmp(tmpname,filename)==0) break;
	}
	/* Error during search*/
	if (search_res==-1)
	{
		return -3;
	}
	
	/* File neuanlegen*/
	if (search_res==DIR_END_OF_DIR)
	{
		if (file_createfile(file,&dir,filename)!=0)
		{
			/*Neuanlegen fehlgeschlagen*/
			return -4;
		}
	}
	/* File gefunden und öffnen*/
	else
	{
		//rewrite file
		if (mode=='w') 
		{
			entry.filesize=0;
			fat_unlink_chain(entry.start_cluster);
			fat_write_cluster(entry.start_cluster,FAT_END_OF_CHAIN);
		} 
		file->akt_cluster = entry.start_cluster;
		file->pos=0;
		file->size=entry.filesize;
		file->start_cluster=entry.start_cluster;
		file->cluster_no_in_chain=0;
		file->dir.entry_position=dir.entry_position-1;
		file->dir.start_cluster=dir.start_cluster;
		file->dir.akt_cluster=dir.akt_cluster;
		file->dir.chain_pos=dir.chain_pos;
		if (mode=='a') 
		{
			if (file->size!=0)
				{
				file->pos=file->size-1;
				}
			file_getchar(file);
		} 
	}
	return 0;
}

int file_seek(file_handle *handle,unsigned long pos)
{
	if (handle->size <= pos) return EOF;
	handle->pos=pos;
	return 0;
}


int file_getchar(file_handle *handle)
	{
		int i=0;
		uint8_t *buffer;
		uint8_t sector;
		
		if (handle->pos == handle->size) 
		{
			return EOF;
		}
		
		uint16_t cluster_nr = handle->pos / 512 / vi.sec_per_cluster;
		
		/*Clusterwechsel*/
		if (cluster_nr != handle->cluster_no_in_chain)
		{
			//next Cluster
			if (cluster_nr == handle->cluster_no_in_chain+1)
			{
				handle->cluster_no_in_chain++;
				handle->akt_cluster=fat_get_next_cluster(handle->akt_cluster);
			}
			//other cluster
			else
			{
				handle->cluster_no_in_chain=cluster_nr;
				handle->akt_cluster=handle->start_cluster;
				for (i=0;i<cluster_nr;i++)
				{
					handle->akt_cluster=fat_get_next_cluster(handle->akt_cluster);
				}
			}
		}
		
		/*Aktcluster stimmt nun auf jeden Fall, also Sektor bestimmen und einlesen*/
		sector = ((handle->pos % (vi.sec_per_cluster*vi.bytes_per_Sector)) / vi.bytes_per_Sector);
		uint32_t phys_sec=vi.data_sector + ((handle->akt_cluster-2) * vi.sec_per_cluster) + sector;
		buffer = fat_read_cb(phys_sec);
		/* Byte in Buffer*/
		return *(buffer+(handle->pos++%vi.bytes_per_Sector));
	}

int file_putchar(file_handle *handle,const char c)
	{
		int append=0;
		int i=0;
		uint8_t *buffer;
		uint8_t sector;
		uint16_t tmpcluster;
		uint32_t physsec;
		
		if (handle->pos > handle->size) 
		{
			return EOF;
		}
		if (handle->pos == handle->size) 
		{
			append=1;
		}
		
		uint16_t cluster_nr = handle->pos / 512 / vi.sec_per_cluster;
		
		/*Clusterwechsel*/
		if (cluster_nr != handle->cluster_no_in_chain)
		{
			//next Cluster -> its a speedup for sequential write
			if (cluster_nr == handle->cluster_no_in_chain+1)
			{
				handle->cluster_no_in_chain++;
				tmpcluster = fat_get_next_cluster(handle->akt_cluster);
				if (tmpcluster==FAT_END_OF_CHAIN)
				{
					if (handle->cluster_no_in_chain == handle->size / 512 / vi.sec_per_cluster)
					{
					tmpcluster = fat_add_cluster(handle->akt_cluster);
					}
					else
					{
						/* EndCluster found in Cluster chain -> filesize missmatch */
						return -1;	
					}
				}
				handle->akt_cluster=tmpcluster;
			}
			//other cluster
			else
			{
				handle->cluster_no_in_chain=cluster_nr;
				handle->akt_cluster=handle->start_cluster;
				for (i=0;i<cluster_nr;i++)
				{
					tmpcluster = fat_get_next_cluster(handle->akt_cluster);
					if (tmpcluster==FAT_END_OF_CHAIN)
					{
						if (i==cluster_nr-1)
						{
						tmpcluster = fat_add_cluster(handle->akt_cluster);
						}
						else
						{
							/* EndCluster found in Cluster chain -> filesize missmatch */
							return -1;	
						}
					}
					handle->akt_cluster = tmpcluster;
				}
			}
		}
		
		/*Aktcluster stimmt nun auf jeden Fall, also Sektor bestimmen und einlesen*/
		sector = ((handle->pos % (vi.sec_per_cluster*vi.bytes_per_Sector)) / vi.bytes_per_Sector);
		physsec=vi.data_sector + ((handle->akt_cluster-2) * vi.sec_per_cluster) + sector;
		buffer = fat_read_cb(physsec);
		/* Byte in Buffer*/
		*(buffer+(handle->pos++%vi.bytes_per_Sector))=c;
		fat_write_cb(physsec);
		if (append) handle->size++;
		return 0;
	}

int file_write(file_handle *file,uint8_t *buffer,unsigned int size)
{
	unsigned int i = (unsigned int) buffer;
	unsigned int t = i+size;
	for (;i<t;i++)
	{
		if (file_putchar(file,*((char*)i))!=0)
		{
			return-1;
		}
	}
	return 0;
}

void file_closefile(file_handle *file)
{
	raw_dir_entry *entry;
	raw_dir_entry tmp_entry;
	entry = dir_readdir_raw(&(file->dir));
	sl_memcpy(&tmp_entry,entry,sizeof(raw_dir_entry));
	/*Hilfskonstrukt für MSP430 der nur gerade wörter schreiben kann*/
	uint8_t *hlp = (uint8*) &tmp_entry.filesize;
	*(hlp++) = (uint8_t)file->size;
	*(hlp++) = (uint8_t)(file->size>>8);
	*(hlp++) = (uint8_t)(file->size>>16);
	*(hlp)   = (uint8_t)(file->size>>24);
	dir_write_entry(&(file->dir), &tmp_entry);
	fat_flush_cb();
}
	
int file_createfile(file_handle *file,dir_handle *handle,const char *path)
{
	int a,b;
	uint16_t cluster;
	uint16_t pathlen;
	raw_dir_entry tmp_entry;
	raw_dir_entry *entry;
	
	/* Filenamen und Ext trennen*/
	pathlen=sl_strlen(path);
	for (a=pathlen-1;a>=0;a--)
		if (path[a]=='.') break;
	char fext[pathlen-a];
	sl_memcpy(fext,path+a+1,pathlen-a);
	b=pathlen-a;
	a--;
	for (; a >= 0; a--)
		if (path[a] == '/') break;
	char filename[pathlen-a-b];
	sl_memcpy(filename, path+a+1, pathlen-a-b-1);
	filename[pathlen-a-b-1]=0;
			
	/* Freien Dir Eintrag suchen*/
	handle->entry_position = 0;
	while(1)
	{
		entry = dir_readdir_raw(handle);
		if (entry == NULL)
		{
			return -1;
			///sl_printf("Directory Full!!\n");	
		}
		if (entry->filename[0] == '\0' || entry->filename[0] == DIR_ENTRY_IS_FREE) break;
		handle->entry_position++;
	}
	/* Fat EIntrag belegen*/
	cluster=fat_add_cluster(0);
	
	// Entry füllen
	int i=0;
	for (i=0;i<8;i++)
		tmp_entry.filename[i]=DIR_NAME_TERMINATOR;
	for (i=0;i<3;i++)
		tmp_entry.extension[i]=DIR_NAME_TERMINATOR;
	sl_memcpy(tmp_entry.filename,filename,strlen(filename));
	sl_memcpy(tmp_entry.extension,fext,strlen(fext));
	/*Hilfskonstrukt für MSP430 der nur gerade wörter schreiben kann*/
	uint8_t *hlp = (uint8*) &tmp_entry.filesize;
	*(hlp++) = 0;
	*(hlp++) = 0;
	*(hlp++) = 0;
	*(hlp)   = 0;
	
	hlp = (uint8*) &tmp_entry.cluster;
	*(hlp++) = cluster;
	*(hlp++) = cluster>>8;

	tmp_entry.flags.asbyte=0;
	dir_write_entry(handle, &tmp_entry);
	
	file->akt_cluster=cluster;
	file->cluster_no_in_chain=0;
	file->dir.akt_cluster=handle->akt_cluster;
	file->dir.chain_pos=handle->chain_pos;
	file->dir.entry_position=handle->entry_position;
	file->dir.start_cluster=handle->start_cluster;
	file->pos=0;
	file->size=0;
	file->start_cluster=cluster;
	return 0;
}
	
#endif

	
