/* Copyright (C) 2005, 2006, 2007, 2008 by Thomas Hillebrandt and Heiko Will

This file is part of the Micro-mesh SensorWeb Firmware.

Micro-Mesh is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3, or (at your option)
any later version.

Micro-Mesh is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Micro-Mesh; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

/* As a special exception, if you include this header file into source
   files compiled by GCC, this header file does not by itself cause
   the resulting executable to be covered by the GNU General Public
   License.  This exception does not however invalidate any other
   reasons why the executable file might be covered by the GNU General
   Public License.  */

#include "config.h"

//TODO: Port to virtual drivers
//TODO: Implement a bulk write function
// Look at /src/<your_architecture>/includes/config.h
#ifdef INCLUDE_SDCARD

#ifndef DRIVERSD_H_
#define DRIVERSD_H_

#include "type.h"

// The IO-Pin for the Chipselect of the Display
#define SD_CS_PIN 	BIT20 	// Chipselect pin


//-------------------------------------------------------------------------------------------------------
//  Spi1SetSpeedKhz(uint16_t speed)
//
//  DESCRIPTION:
//		This functions re-inits the SPI-Bus on the givven speed. 
//
//  ARGUMENTS:
//      uint16_t speed
//          The new Speed in kHz
//  RETURN VALUE:
//      int
//          Returns 0 is successfull.
//
//---------------------------------------------------------------------------------------------------
int Spi1SetSpeedKhz(uint16_t speed);

//-------------------------------------------------------------------------------------------------------
//  int LoadSdDriver(void)
//
//  DESCRIPTION:
//		This functions initializes the hardware (mainly the spi Bus). 
//
//  RETURN VALUE:
//      int
//          Returns 0 is successfull.
//
//---------------------------------------------------------------------------------------------------
extern int LoadSdDriver(void);


//-------------------------------------------------------------------------------------------------------
//  void SdSelect(void)
//
//  DESCRIPTION:
//		Does the Chipselect on the SPI-Bus. 
//
//---------------------------------------------------------------------------------------------------
void SdSelect(void);


//-------------------------------------------------------------------------------------------------------
//  void SdUnselect(void)
//
//  DESCRIPTION:
//		Releases the chipselect on the SPI-Bus. 
//
//---------------------------------------------------------------------------------------------------
void SdUnselect(void);


//-------------------------------------------------------------------------------------------------------
//  void SdRecvN(unsigned char * pbBuf, int iLen)
//
//  DESCRIPTION:
//		This functions implemets a bulk transfer from the sd-card. 
//
//  ARGUMENTS:
//      nsigned char * pbBuf
//          The Buffer to transfer the data to.
//		int iLen
//			The amount of Bytes to receive.
//			TODO: check Datatype. 
//
//---------------------------------------------------------------------------------------------------
extern void SdRecvN(unsigned char * pbBuf, int iLen);


//-------------------------------------------------------------------------------------------------------
//  unsigned char SdSend(unsigned char data)
//
//  DESCRIPTION:
//		This function sends&receives a single Byte to the SPI-Card. 
//		Transfer 0xff if you want to read a byte
//
//  ARGUMENTS:
//      unsigned char data
//          The Byte to send.
//  RETURN VALUE:
//      unsigned char 
//          Returns the received byte.
//
//---------------------------------------------------------------------------------------------------
extern unsigned char SdSend(unsigned char data);

//-------------------------------------------------------------------------------------------------------
//  int Switch2Sd(void)
//
//  DESCRIPTION:
//		This function Reinitializes the whole SPI-BUS. It must be called by the user, if any devices
//		which share ports or registers with the SPI from the SD-Card was used after the last switch2Sd()!
//		TODO: Virtual Devicedriver will eliminate this.
//
//  RETURN VALUE:
//      unsigned char 
//          Returns 0 if successfull.
//
//---------------------------------------------------------------------------------------------------
int Switch2Sd(void);

#endif /*DRIVERSD_H_*/
#endif
