/* Copyright (C) 2005, 2006, 2007, 2008 by Thomas Hillebrandt and Heiko Will

This file is part of the Micro-mesh SensorWeb Firmware.

Micro-Mesh is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3, or (at your option)
any later version.

Micro-Mesh is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Micro-Mesh; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

/* As a special exception, if you include this header file into source
   files compiled by GCC, this header file does not by itself cause
   the resulting executable to be covered by the GNU General Public
   License.  This exception does not however invalidate any other
   reasons why the executable file might be covered by the GNU General
   Public License.  */

#ifndef DRIVER_SYSTEM_H_
#define DRIVER_SYSTEM_H_

#include "config.h"

/* Firmware events*/

typedef struct state_i state_info;

struct state_i
{
	unsigned int POWERDOWN:1	;	// Powerdown is active
	unsigned int TIMER_ACTIVE:1	;	// Timer is active
} state_i;

// Main firmware event flags
typedef struct eventFlags
{
	union
	{
	struct Bits
	{
		unsigned int UART_EDGE_DETECT	: 1;	// UART start edge detection
		unsigned int TIMER				: 1;	// Timer fired
		unsigned int USER_TIMER			: 1;	// Usertimer fired
		unsigned int char_in			: 1;	// character in inbuffer of stdout
	}Bits;
		int Int;
	};
}eventFlags;

// Struct to maintain the state of shared hardware
typedef struct h_config
{
	unsigned int DEBUG_UART_ACTIVE	: 1;	// UART is active for debug
	unsigned int DISPLAY_ACTIVE		: 1;	// Display is active
	unsigned int SDCARD_ACTIVE		: 1;	// SDCard is active
} h_config;




/* misc */

/* Used to init srand */
unsigned int getADCSample(void);

/* Interrupt Handling */

unsigned disableIRQ(void);
unsigned enableIRQ(void);

/* Power Management */
void PowerOff(void);
void GoPowerDown(void);
void GoIdle(void);
void McuReset(void);
#ifdef INCLUDE_BATTERY
float ReadBatteryVoltage(void);
#endif

/* LEDS */

void Led1On(void);
void Led1Off(void);
void Led1Toggle(void);
void Led2On(void);
void Led2Off(void);
void Led2Toggle(void);
void Led3On(void);
void Led3Off(void);
void Led3Toggle(void);
void Led4On(void);
void Led4Off(void);
void Led4Toggle(void);

//-------------------------------------------------------------------------------------------------------
//  void init_clks(void)
//
//  DESCRIPTION:
//		This is inits the clocksystem at startup. Some architektures may call this after a wakeup 
//		from a sleep Mode!
//
//---------------------------------------------------------------------------------------------------
void init_clks(void);


//-------------------------------------------------------------------------------------------------------
//  extern void init_ports(void)
//
//  DESCRIPTION:
//		Is called directly after the PowerOnReset and used to init all Ports of the uC.
//		Most drivers overwrite this settings! 
//---------------------------------------------------------------------------------------------------
extern void init_ports(void);


//-------------------------------------------------------------------------------------------------------
//  void alarm(void)
//
//  DESCRIPTION:
//		Called by Firmware is some peripherals on local busses aren't responding.
//		Could do a hardware reset or an endless loop and debug-output
//
//---------------------------------------------------------------------------------------------------
extern void alarm(void);



// -----------------------------------------------------------------------
// Global Vars of the firmware
// -----------------------------------------------------------------------
extern volatile eventFlags mEventFlags;
extern volatile state_info system_state;
extern volatile h_config hardware_config;

#endif /*DRIVER_SYSTEM_H_*/
