/* Copyright (C) 2005, 2006, 2007, 2008 by Thomas Hillebrandt and Heiko Will

This file is part of the Micro-mesh SensorWeb Firmware.

Micro-Mesh is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3, or (at your option)
any later version.

Micro-Mesh is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Micro-Mesh; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

/* As a special exception, if you include this header file into source
   files compiled by GCC, this header file does not by itself cause
   the resulting executable to be covered by the GNU General Public
   License.  This exception does not however invalidate any other
   reasons why the executable file might be covered by the GNU General
   Public License.  */

#ifndef DRIVER_USERTIMER_H_
#define DRIVER_USERTIMER_H_
#include "type.h"

extern const int timerspeed; // The number of internal Ticks during a Second.
extern const int usertimer_histeresis; 	// The tolerance of the usertimer. 
										// If this is to low and many usertimers proc very close to each other, 
										// some events may lost!
										// The value is relativ to the timerspeed!

// -----------------------------------------------------------------
// userTimer-Api internal Functions (normaly used by usertimer.c)
// -----------------------------------------------------------------

//-------------------------------------------------------------------------------------------------------
//  void load_usertimer_driver(callback_t of,callback_t ft)
//
//  DESCRIPTION:
//		Initializes the timer Hardware. This is called frome the init_user_timer() function
//		and mustn't called by hand in the micro-mesh firmware.
//
//  ARGUMENTS:
//      callback_t ft
//          The callback for any proced usertimer. Called from an interrupt.
//		callback_t of
//			The callback for an overflow of the timer register (Depends on timerspeed and register
//			width if is more than a stub.
//---------------------------------------------------------------------------------------------------
extern void load_usertimer_driver(callback_t of,callback_t ft);


//-------------------------------------------------------------------------------------------------------
//  void setNextUserTimer(unsigned int ticks)
//
//  DESCRIPTION:
//		Sets the *absolute* ticks *offset* of the next usertimer to proc.
//		On 16 Bit Machines, like the msp430 this is only called when the next timer
//		is in the same 16Bit offset range like the timer register. 
//
//  ARGUMENTS:
//      unsigned int ticks
//          The absolute tickcount to wait for.
//
//---------------------------------------------------------------------------------------------------
extern void setNextUserTimer(unsigned int ticks);


//-------------------------------------------------------------------------------------------------------
//  void disableUserTimer(void)
//
//  DESCRIPTION:
//		Disables the usertimer logic. All tick counters won't stop. But no timer will proc.
//
//---------------------------------------------------------------------------------------------------
extern void disableUserTimer(void);

//-------------------------------------------------------------------------------------------------------
//  unsigned int read_usertimer_counter(void)
//
//  DESCRIPTION:
//		Returns the value of the timer register. 
//
//  RETURN VALUE:
//      unsigned int
//          The actual value of the timer register.
//---------------------------------------------------------------------------------------------------
extern unsigned int read_usertimer_counter(void);

#endif /*DRIVER_USERTIMER_H_*/
