/* Copyright (C) 2005, 2006, 2007, 2008 by Thomas Hillebrandt and Heiko Will

This file is part of the Micro-mesh SensorWeb Firmware.

Micro-Mesh is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3, or (at your option)
any later version.

Micro-Mesh is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Micro-Mesh; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

/* As a special exception, if you include this header file into source
   files compiled by GCC, this header file does not by itself cause
   the resulting executable to be covered by the GNU General Public
   License.  This exception does not however invalidate any other
   reasons why the executable file might be covered by the GNU General
   Public License.  */

#ifndef SMALLIB_H_
#define SMALLIB_H_

#include <stdarg.h>
#include <string.h>

//-------------------------------------------------------------------------------------------------------
//	int sl_getchar(int c)
//
//	DESCRIPTION:
//		A lightweight implementation of getchar(). Refer to stdlib doku for further information.
//		Reads in a char from dev/stdin.
//		Returns 0 if there is no char!
//-------------------------------------------------------------------------------------------------------
int sl_getchar();

//-------------------------------------------------------------------------------------------------------
//	int sl_putchar(int c)
//
//	DESCRIPTION:
//		A lightweight implementation of putchar(). Refer to stdlib doku for further information.
//		Prints out a char on dev/stdout
//
//-------------------------------------------------------------------------------------------------------
int sl_putchar(int c);


//-------------------------------------------------------------------------------------------------------
//	int sl_puts(const char *astring)
//
//	DESCRIPTION:
//		A lightweight implementation of puts(). Refer to stdlib doku for further information.
//		Calls void _putchar(int c). You have to implement this to print on your stdout!
//
//-------------------------------------------------------------------------------------------------------
int sl_puts(const char *astring);


//-------------------------------------------------------------------------------------------------------
//	int sl_printf(const char *format, ...)
//
//	DESCRIPTION:
//		A lightweight implementation of printf(). Refer to stdlib doku for further information.
//		Calls void _putchar(int c). You have to implement this to print on your stdout!
//
//-------------------------------------------------------------------------------------------------------
int sl_printf(const char *format, ...);


//-------------------------------------------------------------------------------------------------------
//	sl_sprintf(char *out, const char *format, ...)
//
//	DESCRIPTION:
//		A lightweight implementation of sprintf(). Refer to stdlib doku for further information.
//
//-------------------------------------------------------------------------------------------------------
int sl_sprintf(char *out, const char *format, ...);


//-------------------------------------------------------------------------------------------------------
//	char *sl_strcpy(char *str1, const char *str2)
//
//	DESCRIPTION:
//		A lightweight implementation of strcpy(). Refer to stdlib doku for further information.
//
//-------------------------------------------------------------------------------------------------------
char *sl_strcpy(char *str1, const char *str2);


//-------------------------------------------------------------------------------------------------------
//	void *sl_memcpy(void *str1, const void *str2, size_t n)
//
//	DESCRIPTION:
//		A lightweight implementation of memcpy(). Refer to stdlib doku for further information.
//
//-------------------------------------------------------------------------------------------------------
void *sl_memcpy(void *str1, const void *str2, size_t n);


//-------------------------------------------------------------------------------------------------------
//	size_t sl_strlen(const char *str)
//
//	DESCRIPTION:
//		A lightweight implementation of strlen(). Refer to stdlib doku for further information.
//
//-------------------------------------------------------------------------------------------------------
size_t sl_strlen(const char *str);


//-------------------------------------------------------------------------------------------------------
//	int sl_strcmp(const char *str1, const char *str2)
//
//	DESCRIPTION:
//		A lightweight implementation of strcmp(). Refer to stdlib doku for further information.
//
//-------------------------------------------------------------------------------------------------------
int sl_strcmp(const char *str1, const char *str2);


//-------------------------------------------------------------------------------------------------------
//	char *sl_strcat(char *str1, const char *str2)
//
//	DESCRIPTION:
//		A lightweight implementation of strcat(). Refer to stdlib doku for further information.
//
//-------------------------------------------------------------------------------------------------------
char *sl_strcat(char *str1, const char *str2);


//-------------------------------------------------------------------------------------------------------
//	int sl_atoi(char const* str)
//
//	DESCRIPTION:
//		A lightweight implementation of atoi(). Refer to stdlib doku for further information.
//
//-------------------------------------------------------------------------------------------------------
int sl_atoi(char const* str);


//-------------------------------------------------------------------------------------------------------
//	int sl_abs(int c)
//
//	DESCRIPTION:
//		A lightweight implementation of abs(). Refer to stdlib doku for further information.
//
//-------------------------------------------------------------------------------------------------------
int sl_abs(int c);

//-------------------------------------------------------------------------------------------------------
//	long sl_rand (void)
//
//	DESCRIPTION:
//		A lightweight implementation of rand(). Refer to stdlib doku for further information.
//
//-------------------------------------------------------------------------------------------------------
long sl_rand (void);

//-------------------------------------------------------------------------------------------------------
//	void sl_srand(long s)
//
//	DESCRIPTION:
//		A lightweight implementation of srand(). Refer to stdlib doku for further information.
//
//-------------------------------------------------------------------------------------------------------
void sl_srand(long s);

#endif /*SMALLIB_H_*/
