/* Copyright (C) 2005, 2006, 2007, 2008 by Thomas Hillebrandt and Heiko Will

This file is part of the Micro-mesh SensorWeb Firmware.

Micro-Mesh is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3, or (at your option)
any later version.

Micro-Mesh is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Micro-Mesh; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include "helpers.h"
#include "smallib.h"
#include "type.h"

// Random number between 0 and max-1
unsigned int irand(unsigned int max)
{
	return (unsigned int) sl_abs(sl_rand() % max);
}

uint16_t digits(uint32_t n)
{
    uint16_t digits = (n == 0) ? 1 : 0;
    while(n)
    {
        digits++;
        n/=10;
    }
    return digits;
}



char * long2string(char * buf, long val, char pad, char padchar)
{
	int i;
	char neg = 0;
	char tmp_buf[21];

	if(val<0) {
		val = -val;
		neg = 1;
	}

	for(i=0; val>0; val/=10)
		tmp_buf[i++] = (val % 10) + '0';

	if(i==0)
		tmp_buf[i++] = '0';

	if(neg) {
		if(padchar) {
			tmp_buf[i++] = '-';
			while(i<pad)
				tmp_buf[i++] = ' ';
		}
		else {
			while(i<pad-1)
				tmp_buf[i++] = '0';
			tmp_buf[i++] = '-';
		}
	}
	else
		while(i<pad)
			tmp_buf[i++] = padchar ? ' ' : '0';

	while(i>0)
		*buf++ = tmp_buf[--i];

	*buf = 0;

	return buf;
}

void float2string(char * buf, double f, char d, char pad, char padchar)
{
	char * p;
	const double powers[] = {1.0,10.0,100.0,1000.0,10000.0,100000.0};

	/* round */
	f = f>=0.0 ? (f*powers[(unsigned int)d]+0.5)/powers[(unsigned int)d] : (f*powers[(unsigned int)d]-0.5)/powers[(unsigned int)d];

	/* special case */
	if (f<0.0 && f > -1.0) {
		p = long2string(buf,-1,pad,padchar);
		*(p-1) = '0';
	}
	else
		p = long2string(buf,f,pad,padchar);

	*p++ = '.';

	if (f<0.0)
		f = -f;
	
	f -= (long) f;
	f *= powers[(unsigned int)d];
	long2string(p,(long)f,d,0);
}
