/* Copyright (C) 2005, 2006, 2007, 2008 by Thomas Hillebrandt and Heiko Will

This file is part of the Micro-mesh SensorWeb Firmware.

Micro-Mesh is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3, or (at your option)
any later version.

Micro-Mesh is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Micro-Mesh; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include "config.h"

#ifdef INCLUDE_DISPLAY

#include "driver_display.h"
#include "driver_timer.h"
#include "driver_controller.h"   // LPC I/O definitions

// Pin definitions
#define LCD_CS    BIT24;
#define LCD_RESET BIT23;
#define LCD_SCK   BIT17;
#define LCD_DI    BIT19;
#define LCD_BL    BIT16;

void LoadDisplayDriver (void)
{
	  SSPCR1 = 0x0;		// Disable SPI
	  PINSEL1 |= BIT3+BIT7;
	  PINSEL1 &= ~(BIT5+BIT9); // no csel and Miso
	  FIO1DIR |= LCD_CS;
	  FIO1SET |= LCD_CS;
	  SSPCR0  = 0x00C8;      // cpol = 1, cpha = 1, 9BIT
	  SSPCPSR =  2;           // prescale
	  SSPCR1  = 2;           // enable
	
	  DisplayBacklightOn();
	  _wait(100);
	  FIO0DIR |= LCD_SCK;
	  FIO0DIR |= LCD_DI;
	  FIO0SET |= LCD_SCK;
	  FIO0SET |= LCD_DI;

	  // reset display 
	  FIO1DIR |= LCD_RESET;
	  FIO1CLR |= LCD_RESET;
	  _wait(100);
	  FIO1SET |= LCD_RESET;

}

void Switch2Display(void)
{
	SSPCR1 = 0x0;		// Disable SPI
	PINSEL1 |= BIT3+BIT7;
	PINSEL1 &= ~(BIT5+BIT9); // no csel and Miso
	SSPCR0  = 0x00C8;      // cpol = 1, cpha = 1, 9BIT
	SSPCPSR =  2;           // prescale
	SSPCR1  = 2;           // enable
}

void glcdDisplayCommand(int dat){
  FIO1CLR |= LCD_CS;
  SSPDR  = dat & ~0x0100;// & ~0x00000100;   // send next SPI channel 0 data
  while ((SSPSR & 0x10)) ;     // wait for transfer completed
  FIO1SET |= LCD_CS;
}

inline void DisplayBacklightOff(void)
{
	FIO1CLR=LCD_BL; //Hintergrundbeleuchtung an
}

inline void DisplayBacklightOn(void)
{
	FIO1SET=LCD_BL; //Hintergrundbeleuchtung an
}

void glcdDisplayData(int dat){
  FIO1CLR |= LCD_CS;
  SSPDR  = dat | 0x0100;// & ~0x00000100;   // send next SPI channel 0 data
  while ((SSPSR & 0x10)) ;     // wait for transfer completed
  FIO1SET |= LCD_CS;
}

#endif

