/* Copyright (C) 2005, 2006, 2007, 2008 by Thomas Hillebrandt and Heiko Will

This file is part of the Micro-mesh SensorWeb Firmware.

Micro-Mesh is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3, or (at your option)
any later version.

Micro-Mesh is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Micro-Mesh; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include "driver_timer.h"
#include "driver_controller.h"
#include "armVIC.h"
#include "type.h"

void __attribute__ ((interrupt("IRQ"))) timer0_isr(void);

static callback_t timer_p;

void driver_timer_load(callback_t timer_proc)
{
  timer_p = timer_proc; 
  T0TCR = 0;                  // Disable timer 0.
  T0PR  = 3000;				  // Prescaler is set to relevant pclk , counter is incremented every T0PR tact.
  T0CCR = 0;                  // Capture is disabled.
  T0EMR = 0;                  // No external match output.
  T0TC=0;
  T0TCR = BIT0;          	  // Enable timer 0.
  
  disableIRQ();					// Disable all interrupts
  VICIntEnable = BIT4; 			// Enable Interrupthandling for Timer0
  VICVectCntl3 = 4 + BIT5;		// Assign Timer0 to IRQ Slot 3
  VICVectAddr3 = timer0_isr;	// Assing Isr Address
  enableIRQ();	
}

static volatile unsigned mticks;

uint32_t *TICKS(void)
{
	mticks=T0TC;
	return &mticks;	
}

unsigned int get_timer_count(void)
{
	return T0TC;
}

void _wait(unsigned int num)
{
	unsigned int tmp=*TICKS()+num;
	while (*TICKS()>tmp);
	while (*TICKS()<tmp);
}

void __attribute__ ((interrupt("IRQ"))) timer0_isr(void)
{
	
	timer_p();
	T0IR = 0xff;      // clear all interrupt of Timer0
	VICVectAddr=0;  //Dummy write to signal end of interrupt
}

void set_next_timer_value(unsigned int ticks)
{
	T0MR0=ticks;// Timer interrupt set match
	T0MCR|=BIT0;	// Timer interrupt set on T0TC==MR0
}

void disable_timer(void)
{
	T0MCR&=~BIT1;// Int aus
}
