/* Copyright (C) 2005, 2006, 2007, 2008 by Thomas Hillebrandt and Heiko Will

This file is part of the Micro-mesh SensorWeb Firmware.

Micro-Mesh is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3, or (at your option)
any later version.

Micro-Mesh is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Micro-Mesh; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include "driver_uart0.h"
#include "driver_controller.h"
#include "type.h"
#include "board.h"
#include "smallib.h"

void __attribute__ ((interrupt("IRQ"))) Uart0IntHandler(void) ;

/* External Firmware invokes*/
extern void dm_activate(driver* d);

/* Private functions */
static void uart_init(driver* this);
static inline void clock_init(driver* this);
static void uart0_put_char(driver* this,int character);

/* Private fields*/
static driver_serial *active;

/* Public definitions */

driver_serial* uart0_new(void* char_in_ref,driver_cb_cin_t char_in,driver_serial* this)
{
	this->driver.base.init= uart_init;
	this->driver.activate= uart_init;
	this->char_in = char_in;
	this->put_char = uart0_put_char;
	this->char_in_ref = char_in_ref;
	this->driver.base.active = 0;
	this->driver.base.awake = 1;
	return this;
} 

/* Private definitions */
void uart0_put_char(driver* this,int character)
{	
	if (!this->active) {
		//dm_activate((driver*)this);
		uart_init(this);
		}
	/* Hier fragen wir das "Transmitter Holding Register Empty"-Bit
	   im "Line Status Register" ab und ermitteln, ob sich noch ein Byte im 
	   Sendepuffer befindet (der Sendepuffer ist ein FIFO-Stack) */
	while (!(U0LSR & BIT5)); /* Warten, bis der Sendepuffer geleert ist */
	/* Neuen Wert in das "Transmitter Holding Register" schreiben */
	U0THR = character;
}

void __attribute__ ((interrupt("IRQ"))) Uart0IntHandler(void) 
{ 
	int x;
	if (active!=NULL){
		active->char_in(active->char_in_ref,U0RBR);
	}
	x = U0IIR;
	x = U0LSR;
	VICVectAddr = 0; // Acknowledge Interrupt (rough?) 
} 


inline void clock_init(driver *this)
{
	driver_serial *me = (driver_serial*) this;
	// Divisor Latch Access Bit setzen (DLAB)
	U0LCR |= (1<<7);
	
	// Teiler = PCLK / (baudrate * 16)
	// Untere 8 Bit des Teilers
	U0DLL = (PCLK / (me->baudrate * 16)) & 0xFF;
	// Obere 8 Bit des Teilers
	U0DLM = ((PCLK / (me->baudrate * 16)) & 0xFF00)>>8;
		
	// DLAB wieder löschen
	U0LCR &= ~(1<<7);
}

void uart_init(driver *this)
{
	  PINSEL0 &= ~((0x3<<2) | (0x3<<0)); // Pin-Funktion l�schen (zur Sicherheit)
	  //         rxd 0     txd 0
	  PINSEL0 |= (1<<2) | (1<<0);        // Pinfunktion zuweisen
		
	  clock_init(this);
		
	  // 9 bit, 1 stop bit, keine parit�t
	  U0LCR = (1<<1) | 1;
	 
	  // UART0-FIFO aktivieren
	  U0FCR = BIT0+BIT1+BIT2;
	  
	  VICVectAddr4 = (unsigned long) Uart0IntHandler; // set interrupt vector in Slot 1 
	  VICVectCntl4 = BIT5 + 6; // use it for UART0 Interrupt: 
	  VICIntEnable |= (BIT6); // Enable UART0 Interrupt 
	  U0IER = BIT0; 
}

void deactivate(driver_serial* this)
{
	this->driver.base.active=0;
	VICIntEnable &= !BIT6; // Enable UART0 Interrupt 
}



