/* Copyright (C) 2005, 2006, 2007, 2008 by Thomas Hillebrandt and Heiko Will

This file is part of the Micro-mesh SensorWeb Firmware.

Micro-Mesh is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3, or (at your option)
any later version.

Micro-Mesh is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Micro-Mesh; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include "driver_timer.h"
#include "driver_controller.h"
#include "armVIC.h"
#include "type.h"

static int sched_init=0;

void __attribute__ ((interrupt("IRQ"))) timer1_isr(void);

void scheduler_init(void)
{
  T1TCR = 0;                  // Disable timer 1.
  T1PR  = 3000;				  // Prescaler is set to relevant pclk , counter is incremented every T0PR tact.
  T1CCR = 0;                  // Capture is disabled.
  T1EMR = 0;                  // No external match output.
   
  disableIRQ();					// Disable all interrupts
  VICIntEnable = BIT5; 			// Enable Interrupthandling for Timer0
  VICVectCntl6 = 5 + BIT5;		// Assign Timer0 to IRQ Slot 3
  VICVectAddr6 = (unsigned long) timer1_isr;	// Assing Isr Address
  enableIRQ();	
}

volatile int sheduler_back;
volatile int sheduler_main_sr;
volatile int sheduler_main;
volatile int userapp_sp;

void __attribute__ ((interrupt("IRQ"))) __attribute__ ((naked)) timer1_isr(void)
{
	asm volatile(
			"STMDB	SP!, {R0-R12}	\n\t"	   // Alle Register der Userapp sichern
			"NOP		\n\t"
			
			"MRS R0, SPSR \n\t"
			"STMDB	SP!, {R0}	\n\t"	   // Statusregister der Userapp sichern
			"NOP		\n\t"
			
			"MOV R0,R14 \n\t"
			"STMDB	SP!, {R0}	\n\t"	   // Rücksprungadresse in die Userapp sichern
			"NOP		\n\t"
			
			"MOV R0, #2		\n\t"	   // Unterbrechen der Userapp mit Status=2 kennzeichnen
			"LDR R1, =sheduler_back \n\t"
			"STR R0,[R1] \n\t"	  
			
			"LDR R1, =userapp_sp \n\t"
			"STR R13, [R1] \n\t" 	   // Stackpointer sichern
				
			"LDR R1, = sheduler_main \n\t"
			"LDR R14, [R1] \n\t"  // Rücksprungadresse nach R14
			
			"LDR R1, =sheduler_main_sr \n\t"
			"LDR R1, [R1] \n\t"// Statusregister zurücksichern
			"MSR SPSR, R1 \n\t"
			
			"mov	r3, #-536870912	\n\t"
			"add	r3, r3, #32768	\n\t"
			"mov	r1, #255 \n\t"
			"str	r1, [r3] \n\t"
			"mov	r2, #0	\n\t"
			"mvn	r3, #0	\n\t"
			"str	r2, [r3, #-4047] \n\t"
			
			"SUBS PC, R14, #0 \n\t" // Interrupt wird abgeschlossen und r14 und CPSR wiederhergestellt
	); 
}

void disable_sheduler(void)
{
	T1MCR&=~BIT1;// Int aus
}

void activate_sheduler(void)
{
	if (!sched_init) scheduler_init();
	T1TC = 0;
	T1TCR = BIT0;   // Enable timer 0.
	T1MR0 = 1100;	// Timer interrupt set match
	T1MCR |= BIT0;	// Timer interrupt set on T1TC==MR1
}

void __attribute__ ((naked)) switch_userapp(void)
{
	asm(
		"LDR R1, =userapp_sp \n\t"
		"LDR SP, [R1] \n\t" 	   // Stackpointer wiederherstellen
			
		"LDR R1, =sheduler_main_sr \n\t" // Statusregister der Main sichern
		"MRS R0, CPSR \n\t"
		"STR R0, [R1] \n\t"

		"LDMIA	SP!, {R0} \n\t" // Restore Return Address
		"nop \n\t"
		"MOV R14, R0 \n\t"
			
		"LDMIA	SP!, {R0} \n\t" // Restore Statusregister
		"nop \n\t"
		"MSR CPSR,R0 \n\t"
			
		"LDMIA	SP!, {R0-R12} \n\t" // Restore Registers
		"nop \n\t"
		"SUB R14,R14,#4 \n\t"
		"BX LR \n\t"
	);
}

void __attribute__ ((naked)) start_user_thread(callback_t thread)
{	
	asm volatile(
		"LDR R2, =sheduler_main \n\t" // Rücksprungadresse der Main sichern
		"STR R14, [R2] \n\t"

		"LDR R2, =sheduler_main_sr \n\t" // Statusregister der Main sichern
		"MRS R1, CPSR \n\t"
		"STR R1, [R2] \n\t"
			
		"STMDB	SP!, {R0}	\n\t"	 		 // Parameter sichern
		"BL activate_sheduler \n\t"
		"LDMIA	SP!, {R0} \n\t" // Restore Parameter

		"MOV R14, PC \n\t"	
		"MOV PC, R0 \n\t"			 // In den Thread springen
									 // Thread hat geendet

//		"call #disable_sheduler \n\t"	// Sheduler Timer anhalten
//		"cmp #0,sheduler_back \n\t"	// Wurde der Thread unterbrochen
//		"jz end \n\t"				// Wenn nein, war dies ein normaler funktionsaufruf
//		"mov #0,sheduler_back \n\t" // Wenn ja, Den Zustand vom vorm *letzten* Einsprung wiederherstellen
//		"mov sheduler_main_sr, r2 \n\t"
//		"end: \n\t"
//		"mov sheduler_main, r0 \n\t" // Zurückspringen
		);
}

void continue_thread(void)
{
	sheduler_back=2;
}

void sleep(unsigned int secs)
{
//	asm(
//			"push r2 \n\t"
//			"push r15 \n\t"			 // Thread Startadresse sichern
//			"call #disable_sheduler \n\t"
//			"mov #1,sheduler_back \n\t"
//			"pop r15 \n\t"			 // Thread Startadresse zurückholen	
//										  // r2 und Sprungadresse der Userapp liegen noch auf dem Stack
//			"push r4 \n\t" 				  // Alle Register der Userapp sichern
//			"push r5 \n\t"					
//			"push r6 \n\t"
//			"push r7 \n\t"
//			"push r8 \n\t"
//			"push r9 \n\t"
//			"push r10 \n\t"
//			"push r11 \n\t"
//			"push r12 \n\t"
//			"push r13 \n\t"
//			"push r14 \n\t"
//			"push r15 \n\t"
//			"mov r1, userapp_sp \n\t" //Stackpointer sichern
//			"push sheduler_main \n\t"	  // Aktuelle Adresse der Main auf den Stack
//	); // Interrupt wird mit Reti abgeschlossen und r2 und rücksprung vom Stack geladen.
//	registerUserTimer(continue_thread, secs);
}
