/* Copyright (C) 2005, 2006, 2007, 2008 by Thomas Hillebrandt and Heiko Will

This file is part of the Micro-mesh SensorWeb Firmware.

Micro-Mesh is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3, or (at your option)
any later version.

Micro-Mesh is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Micro-Mesh; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include <stdio.h>
#include "config.h"
#include "smallib.h"
#include "type.h"
#include "timer.h"
#include "driver_timer.h"
#include "power.h"
#include "helpers.h"
#include "routing.h"
#include "net.h"
#include "command.h"
#include "usertimer.h"
#include "boot.h"
#include "user_pakethandlers.h"
#include "user_apps.h"
#include "driver_system.h"
#include "cc1100_defines.h"
#include "rulesystem.h"


void main_loop(void);

// Applications
void wakeupApplication(void*);


/*************************************************************************
	main
	====
**************************************************************************/
void testapp(void)
{
	char herm[] ="Userapp!";
	int x=0;
	unsigned int y=0;
	while(1)
	{
		//for(y=0;y<500;y++){
			sl_printf("%s: Testapp - x: %d y: %d \n",herm,x++,y);
		//}
		//sleep(10);
	}
}

int main(void)
{
	/* Initialise all device drivers */
	boot();
	/* start routing layer */
	start_routing();
	/* start paket-handlers for user apps */
	init_user_paket_handlers();
	/* start logging to sd card */
	start_logging();
	/* enable interrupts */
	enableIRQ();

	/* Display menu */
	showMenu();

	/* start the mainloop. This never returns */
	/*start_user_thread(testapp);*/

	main_loop();
	return 0;
}


static int wakeupTimerId=0;
static uint32_t wakeupTime;


void main_loop(void) {
	bool b;
	char c;
	int i;

	/* Multitasking mainjump */
	sheduler_main = (unsigned int)main_loop;

	/* Mainloop */
	while (1) {
		disable_sheduler();
		handleRadioEvents();
		b = isPacketHandlingNeeded();
		if (b)
			handlePackets();

		if (mEventFlags.Bits.TIMER) {
			mEventFlags.Bits.TIMER = false;
			for (i = 0; i < FIRED_TIMERS_SIZE; i++) {
				if (fired_timers[i] != NULL) {
					fp_timer tmp = fired_timers[i];
					fired_timers[i] = NULL;
					tmp();
				}
			}
		}

		if (mEventFlags.Bits.char_in) {
			c=sl_getchar();
			execCommand(c);
		}

		if (rflags.RX_REQ)
			continue;

		if (sheduler_back==2)
		{
			activate_sheduler();
			sl_printf("switch back\n");
			switch_userapp();
			continue;
		}

#ifdef DRIVER
		WDTCTL = WDTPW + WDTSSEL + 1 + WDTCNTCL; //Watchdog every 1/4 seconds
#endif
	}
}

// Application for LPM3 wakeup, does nothing
void wakeupApplication(void* none)
{
	return;
}






