/* Copyright (C) 2005, 2006, 2007, 2008 by Thomas Hillebrandt and Heiko Will

This file is part of the Micro-mesh SensorWeb Firmware.

Micro-Mesh is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3, or (at your option)
any later version.

Micro-Mesh is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Micro-Mesh; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include <stdio.h>
#include "net.h"
#include "type.h"
//---------------------------------------------------------------------------
//					Data types and variables of the net layer
//---------------------------------------------------------------------------

typedef struct {
	uint8_t type;
	packet_handler_t handler;
} packet_handler_entry;

static packet_handler_entry handlerTable[HANDLER_TABLE_SIZE];

//---------------------------------------------------------------------------
//					PRIVATE   F U N C T I O N S   of the net layer
//---------------------------------------------------------------------------

static int findHandlerIndex(uint8_t type)
{
	uint8_t i;
	for (i = 0; i < HANDLER_TABLE_SIZE; i++)
	{
		if (handlerTable[i].type == type) break;
	}
	return i != HANDLER_TABLE_SIZE ? i : -1;
}

//---------------------------------------------------------------------------
//					PUBLIC   F U N C T I O N S   of the net layer
//---------------------------------------------------------------------------

void initNet(void)
{
	uint8_t i;
	for (i = 0; i < HANDLER_TABLE_SIZE; i++)
	{
		handlerTable[i].type = 0;
	}
}

void net_packet_handler(packet_layer1_t* packet)
{
	int index = findHandlerIndex(packet->protocol);
	if (index != -1) 
	{
		handlerTable[index].handler(packet->source,packet->data);
	}
}

bool net_send(uint8_t type, uint8_t dest, uint8_t* data)
{
	return sendLayer1Data(data, type, dest);
}

bool registerPacketHandler(uint8_t type, packet_handler_t handler)
{
	if (type < P_DATA) return false;
	int index = findHandlerIndex(type);
	if (index == -1) index = findHandlerIndex(0);
	if (index == -1) return false;
	handlerTable[index].type = type;
	handlerTable[index].handler = handler;
	return true;
}

bool unregisterPacketHandler(uint8_t type)
{
	int index = findHandlerIndex(type);
	if (index != -1)
	{
		handlerTable[index].type = 0;
		handlerTable[index].handler = NULL;
		return true;
	}
	return false;
}
