#ifndef RULESYSTEM_H_
#define RULESYSTEM_H_


/* General Datatypes */

typedef enum states 
{
	st_sensor = 0,
	st_value = 1
}states_t;

typedef enum actions
{
	ac_send = 0,
	ac_log = 1,
	ac_act = 2,
}actions_t;

typedef enum
{
	t_int = 0,
	t_float = 1,
	t_uint = 2,
	t_long = 3
}type_t;

typedef enum binops_t
{
	binop_eq = 0,
	binop_sm = 1,
	binop_gr = 2,
	binop_se = 3,
	binop_ge = 4,
	binop_ne = 5
}binops_t;

typedef struct __attribute__ ((packed)) typed_value_t
{
	union {
	float fval;
	int16_t ival;
	uint16_t uval;
	long lval;
	} value_t;
	uint8_t valtype;
}typed_value_t;

typedef struct __attribute__ ((packed)) expression
{
	typed_value_t value;
	uint8_t func;
} expression;

typedef int (*binop_t)(typed_value_t* a,typed_value_t* b);
typedef int (*thenop_t)(int rule,expression *operation,expression *operand);
typedef typed_value_t* (*evalfunc_t)(typed_value_t* val,typed_value_t *ret);

/* if (EXP OP EXP) then FUNC(EXP,EXP) else FUNC(EXP,EXP) */
struct __attribute__ ((packed)) rule_t{
	uint32_t timestamp; 
	expression if_a;
	expression if_b;
	expression thenoperation;
	expression elseoperation;
	expression thenoperand;
	expression elseoperand;
	uint8_t ifoperation;
	uint8_t check_period;
	uint16_t id;
};

/* forward declaration of rule_t for using it recursive */
typedef struct rule_t rule_t;

/* Protocoll Definitions for sending rules over the air */
#define RULE_ADDRULE 1
#define RULE_DELRULE 2
#define RULE_ADDREPLY 3
#define RULE_DELREPLY 4
#define RULE_GETRULE 5
#define RULE_GETREPLY 6
#define RULE_GETALL 7
#define RILE_GA_REPLY 8

/* First 4 Bytes are the header which is defined as folow:
 * 1st Byte is Command
 * 2nd Byte is Seq No
 * 3rd Byte is rule id for commands like del, addreply, delreply, getrule, getreply
 * 4th Byte is reserved
 * ga_reply sends an 0xff terminated lsit of all rule ids after the header
 */
typedef struct __attribute__ ((packed)) rule_protocoll_t{
	unsigned char command;
	unsigned char seqno;
	unsigned char rule_id;
	unsigned char reserved;
	rule_t rule;
} rule_protocoll_t;


/* Public functions */
int init_rulesystem(void);

#define DEBUGRULESYSTEM 0
#if DEBUGRULESYSTEM
void printrule(rule_t *rule);
#else
	#define printrule(...) 
#endif

int add_rule(rule_t *rule);
int del_rule(uint8_t id);

#endif /*RULESYSTEM_H_*/
