/* Copyright (C) 2005, 2006, 2007, 2008 by Thomas Hillebrandt and Heiko Will

This file is part of the Micro-mesh SensorWeb Firmware.

Micro-Mesh is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3, or (at your option)
any later version.

Micro-Mesh is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Micro-Mesh; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include "config.h"

#ifdef INCLUDE_SDCARD

#include "smallib.h"
#include "sd.h"
#include "type.h"
#include "driver_sd.h"
#include "helpers.h"


static uint8_t mmc_read_csd (uint8_t *Buffer);

bool SdPresent=false;

//############################################################################
//Routine zum Empfangen eines Bytes von der MMC-Karte 
inline uint8_t mmc_read_byte (void)
//############################################################################
{
	uint8_t Byte = 0xff;
	Byte = SdSend(0xff);
	return (Byte);
}

//############################################################################
//Routine zum Senden eines Bytes zur MMC-Karte
inline void mmc_write_byte (uint8_t Byte)
//############################################################################
{
    SdSend(Byte);
}

//############################################################################
//Sendet ein Commando an die MMC/SD-Karte
uint8_t mmc_write_command (uint8_t *cmd)
//############################################################################
{
	uint8_t tmp = 0xff;
	int Timeout = 0x0fff;
	
	//sendet 6 Byte Commando
	uint8_t a;
	for (a = 0;a<6;a++) //sendet 6 Byte Commando zur MMC/SD-Karte
		{
		mmc_write_byte(cmd[a]);
		}

	//Wartet auf ein gültige Antwort von der MMC/SD-Karte
	while (Timeout-- > 0)	
		{
		tmp = mmc_read_byte();
		if (tmp != 0xff)
			break; //MMC hat geantwortet
		}
	return(tmp);
}

/*
* Repeatedly reads the MMC until we get the
* response we want or timeout
*/
uint8_t mmc_response(uint8_t response)
{
	uint32_t count = 0xFFF;
	uint8_t result;
	while( count > 0 )
	{
		result = mmc_read_byte();
		if ( result == response )
		{
			break;
		}
		count--;
	}
	if ( count == 0 )
		return 1; /* Failure, loop was exited due to timeout */
	else
		return 0; /* Normal, loop was exited before timeout */
}

//############################################################################
//Routine zur Initialisierung der MMC/SD-Karte (SPI-MODE)
uint8_t InitSd()
//############################################################################
{
	if (LoadSdDriver()!=0)
		{
			return 3;
		}
	uint16_t Timeout = 0;
	
	uint8_t cmd[] = {0x40,0x00,0x00,0x00,0x00,0x95};
	
	SdSelect();
	uint8_t a=0xff;
	
	//Sendet Commando cmd0 an MMC/SD-Karte
	while ((a = mmc_write_command(cmd)) !=1)
	{
		if (Timeout++ == 0x200)
		{
			return 1; //Abbruch bei Commando0 (Return Code1)
		}
	}
	
	// Wait 8 Clocks
	mmc_read_byte();
	
	//Sendet Commando cmd1 an MMC/SD-Karte
	Timeout = 0;
	cmd[0] = 0x41;//Commando 1
	cmd[5] = 0xff;

	while( mmc_write_command (cmd) !=0)
	{
		if (Timeout++ == 0x200)
			{
			return(2); //Abbruch bei Commando2 (Return Code2)
			}
	}
	SdUnselect();
	
	// Wait 8 Clocks
	mmc_read_byte();
	Spi1SetSpeedKhz(3500);
	SdPresent=true;
	return(0); 
}

//############################################################################
//Routine zum schreiben eines Blocks(512Byte) auf die MMC/SD-Karte
int sd_write_sector (uint32_t addr,uint8_t *Buffer)
//############################################################################
{
	SdSelect();
	uint8_t tmp;
	//Commando 24 zum schreiben eines Blocks auf die MMC/SD - Karte
	uint8_t cmd[] = {0x58,0x00,0x00,0x00,0x00,0xFF}; 
	
	/*Die Adressierung der MMC/SD-Karte wird in Bytes angegeben,
	  addr wird von Blocks zu Bytes umgerechnet danach werden 
	  diese in das Commando eingef�gt*/
	  
	addr = addr << 9; //addr = addr * 512
	
	cmd[1] = ((addr & 0xFF000000) >>24 );
	cmd[2] = ((addr & 0x00FF0000) >>16 );
	cmd[3] = ((addr & 0x0000FF00) >>8 );

	//Sendet Commando cmd24 an MMC/SD-Karte (Write 1 Block/512 Bytes)
	tmp = mmc_write_command (cmd);
	if (tmp != 0)
		{
		//sl_printf("Error writing on writecommand \n");
		return(-1);
		}
	
	mmc_write_byte(0xFF); 
	
	//Sendet Start Byte an MMC/SD-Karte
	mmc_write_byte(0xFE);	
	
	//Schreiben des Blocks (512Bytes) auf MMC/SD-Karte
	uint16_t a=0;
	for (a=0;a<512;a++)
		{
		 SdSend(*(Buffer++));
		}
	
	//CRC-Byte schreiben
	mmc_write_byte(0xFF); //Schreibt Dummy CRC
	mmc_write_byte(0xFF); //CRC Code wird nicht benutzt
	
	// Data Response
	tmp=mmc_read_byte();
	//Wartet auf MMC/SD-Karte Busy
	while ((tmp=mmc_read_byte()) != 0xff){};
	
	//set MMC_Chip_Select to high (MMC/SD-Karte Inaktiv)
	SdUnselect();
	
return(0);
}

//############################################################################
//Routine zum lesen eines Blocks von der Karte
uint8_t mmc_read_block(uint8_t *cmd,uint8_t *Buffer,uint16_t Bytes)
//############################################################################
{	
	SdSelect();
	//Sendet Commando cmd an MMC/SD-Karte
	if (mmc_write_command (cmd) != 0)
			{
			 //sl_printf("couldn't Read Sektor");
			 return 1;
			}
	//Wartet auf Start Byte von der MMC/SD-Karte (FEh/Start Byte)
	if((mmc_response(0xFE))==1)
	{
		SdUnselect();
		//sl_printf("couldn't Read Sektor");
		return 1;
   	}
	//Lesen des Blocks (normal 512Bytes) von MMC/SD-Karte
	uint16_t a;
	for (a=0;a<Bytes;a++)
		{
		Buffer[a] = mmc_read_byte();
		}
	//CRC-Byte auslesen
	mmc_read_byte();//CRC - Byte wird nicht ausgewertet
	mmc_read_byte();//CRC - Byte wird nicht ausgewertet

	//set MMC_Chip_Select to high (MMC/SD-Karte Inaktiv)
	SdUnselect();
	mmc_read_byte();
	return 0;
}

//############################################################################
//Routine zum lesen eines Blocks(512Byte) von der MMC/SD-Karte
int sd_read_sector (uint32_t addr,uint8_t *Buffer)
//############################################################################
{	
	//Commando 16 zum lesen eines Blocks von der MMC/SD - Karte
	uint8_t cmd[] = {0x51,0x00,0x00,0x00,0x00,0xFF}; 
	
	/*Die Adressierung der MMC/SD-Karte wird in Bytes angegeben,
	  addr wird von Blocks zu Bytes umgerechnet danach werden 
	  diese in das Commando eingef�gt*/
	  
	addr = addr << 9; //addr = addr * 512

	cmd[1] = ((addr & (uint32)0xFF000000) >>24 );
	cmd[2] = ((addr & (uint32)0x00FF0000) >>16 );
	cmd[3] = ((addr & (uint32)0x0000FF00) >>8 );
	uint8_t ret;
    ret=mmc_read_block(cmd,Buffer,512);
	return(ret);
}

//############################################################################
//Routine zum lesen des CID Registers von der MMC/SD-Karte (16Bytes)
uint8_t mmc_read_cid (uint8_t *Buffer)
//############################################################################
{
	//Commando zum lesen des CID Registers
	uint8_t cmd[] = {0x4A,0x00,0x00,0x00,0x00,0xFF}; 
	
	mmc_read_block(cmd,Buffer,16);

	return(0);
}

//############################################################################
//Routine zum lesen des CSD Registers von der MMC/SD-Karte (16Bytes)
uint8_t mmc_read_csd (uint8_t *Buffer)
//############################################################################
{	
	//Commando zum lesen des CSD Registers
	uint8_t cmd[] = {0x49,0x00,0x00,0x00,0x00,0xFF};
	
	mmc_read_block(cmd,Buffer,16);

	return(0);
}


//############################################################################
//Routine um die gr��e der MMC zu bestimmen. 
//############################################################################
int BlockDevGetSize(uint32_t *pdwDriveSize)
{
	uint8_t by;
	uint8_t iob[16];
	uint16_t c_size, c_size_mult, read_bl_len;

	mmc_read_csd (iob);
	mmc_write_byte(0xff);

	c_size = iob[6] & 0x03;		// bits 1..0
	c_size <<= 10;
	c_size += (uint16_t) iob[7] << 2;
	c_size += iob[8] >> 6;

	by = iob[5] & 0x0F;
	read_bl_len = 1 << by;

	by = iob[9] & 0x03;
	by <<= 1;
	by += iob[10] >> 7;

	c_size_mult = 1 << (2 + by);

	*pdwDriveSize = (uint32) (c_size + 1) * (uint32) c_size_mult *(uint32) read_bl_len;

	return 0;
}

#endif
