/* Copyright (C) 2005, 2006, 2007, 2008 by Thomas Hillebrandt and Heiko Will

This file is part of the Micro-mesh SensorWeb Firmware.

Micro-Mesh is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3, or (at your option)
any later version.

Micro-Mesh is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Micro-Mesh; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

/* As a special exception, if you include this header file into source
   files compiled by GCC, this header file does not by itself cause
   the resulting executable to be covered by the GNU General Public
   License.  This exception does not however invalidate any other
   reasons why the executable file might be covered by the GNU General
   Public License.  */

#include "config.h"

#ifdef INCLUDE_SDCARD

#ifndef SD_H_
#define SD_H_

#include "type.h"

#define DUMMY 0xff


// Tokens (nessisary because at nop/idle (and CS active) only 0xff is on the data/command line)
#define MMC_START_DATA_BLOCK_TOKEN 		0xfe	// Data token start byte, Start Single Block Read
#define MMC_START_DATA_MULTIPLE_BLOCK_READ 	0xfe	// Data token start byte, Start Multiple Block Read
#define MMC_START_DATA_BLOCK_WRITE		0xfe	// Data token start byte, Start Single Block Write
#define MMC_START_DATA_MULTIPLE_BLOCK_WRITE	0xfc	// Data token start byte, Start Multiple Block Write
#define MMC_STOP_DATA_MULTIPLE_BLOCK_WRITE	0xfd	// Data toke stop byte, Stop Multiple Block Write


// an affirmative R1 response (no errors)
#define MMC_R1_RESPONSE       0x00


// this variable will be used to track the current block length
// this allows the block length to be set only when needed
// unsigned long _BlockLength = 0;

// error/success codes
#define MMC_SUCCESS           0x00
#define MMC_BLOCK_SET_ERROR   0x01
#define MMC_RESPONSE_ERROR    0x02
#define MMC_DATA_TOKEN_ERROR  0x03
#define MMC_INIT_ERROR        0x04
#define MMC_CRC_ERROR         0x10
#define MMC_WRITE_ERROR       0x11
#define MMC_OTHER_ERROR       0x12
#define MMC_TIMEOUT_ERROR     0xFF


// commands: first bit 0 (start bit), second 1 (transmission bit); CMD-number + 0ffsett 0x40
#define MMC_GO_IDLE_STATE 	0x40    //CMD0
#define MMC_SEND_OP_COND 	0x41    //CMD1
#define MMC_READ_CSD 		0x49	//CMD9
#define MMC_SEND_CID 		0x4a    //CMD10
#define MMC_STOP_TRANSMISSION 	0x4c    //CMD12
#define MMC_SEND_STATUS 	0x4d    //CMD13
#define MMC_SET_BLOCKLEN 	0x50	//CMD16 Set block length for next read/write
#define MMC_READ_SINGLE_BLOCK 	0x51	//CMD17 Read block from memory
#define MMC_READ_MULTIPLE_BLOCK 0x52    //CMD18
#define MMC_CMD_WRITEBLOCK 	0x54	//CMD20 Write block to memory
#define MMC_WRITE BLOCK 	0x58    //CMD25
#define MMC_WRITE_MULTIPLE_BLOCK 0x59   //CMD??
#define MMC_WRITE_CSD 		0x5b	//CMD27 PROGRAM_CSD
#define MMC_SET_WRITE_PROT 	0x5c    //CMD28
#define MMC_CLR_WRITE_PROT 	0x5d    //CMD29
#define MMC_SEND_WRITE_PROT 	0x5e    //CMD30
#define MMC_TAG_SECTOR_START 	0x60    //CMD32
#define MMC_TAG_SECTOR_END 	0x61    //CMD33
#define MMC_UNTAG_SECTOR 	0x62    //CMD34
#define MMC_TAG_EREASE_GROUP_START 0x63 //CMD35
#define MMC_TAG_EREASE_GROUP_END 0x64   //CMD36
#define MMC_UNTAG_EREASE_GROUP 	0x65    //CMD37
#define MMC_EREASE 		0x66    //CMD38
#define MMC_READ_OCR 		0x67    //CMD39
#define MMC_CRC_ON_OFF 		0x68    //CMD40


extern bool SdPresent;

extern int sd_read_sector(uint32, uint8_t *);
extern int sd_write_sector(uint32, uint8_t *);
int BlockDevGetSize(uint32_t *pdwDriveSize);
extern uint8_t InitSd (void);

#endif /*SD_H_*/
#endif
