/* Copyright (C) 2005, 2006, 2007, 2008 by Thomas Hillebrandt and Heiko Will

This file is part of the Micro-mesh SensorWeb Firmware.

Micro-Mesh is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3, or (at your option)
any later version.

Micro-Mesh is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Micro-Mesh; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

/* As a special exception, if you include this header file into source
   files compiled by GCC, this header file does not by itself cause
   the resulting executable to be covered by the GNU General Public
   License.  This exception does not however invalidate any other
   reasons why the executable file might be covered by the GNU General
   Public License.  */

#ifndef USERTIMER_H_
#define USERTIMER_H_

#include "type.h"

// --------------------------------------------------------
// Defines
// --------------------------------------------------------

#define MAX_USER_TIMERS (10)	// The maximum number of regsitered usertimers

typedef void (*fp_utimer) (void *state);	// The callback for a proced usertimer

typedef struct fired_timer {
	fp_utimer fp;
	void *state;
} fired_timer;

// --------------------------------------------------------
// Global Vars
// --------------------------------------------------------

extern volatile fired_timer firedUserTimers[MAX_USER_TIMERS]; // The array for all fired usertimers. Only the kernel should write to it!

// --------------------------------------------------------
// Public Functions
// --------------------------------------------------------

//----------------------------------------------------------------------------------------------------
//  void startUserTimer()
//
//  DESCRIPTION:
//		Initialises all usertimer data-structures. Calls driver_usertimer.load_usertimer_driver()
//		to init the Hardware.
//		TODO: Rename to int_user_timer() to match coding conventions
//---------------------------------------------------------------------------------------------------
extern void startUserTimer();

//-------------------------------------------------------------------------------------------------------
//  int registerUserTimer(fp_utimer fp, uint16_t offset)
//
//  DESCRIPTION:
//		This functions registers a nonblocking usertimer. The callback is transfered in the 
//		firedUserTimers array and *not* called directly. So it could be invoked from outside 
//		an interrupt. 
//
//  ARGUMENTS:
//      fp_utimer fp
//          A function pointer to be called if the timer procs.
//		uint16_t offset
//			The number of seconds until the timer procs.
//  RETURN VALUE:
//      int
//          The id of the registered timer. returns 0 on error.
//
//---------------------------------------------------------------------------------------------------
extern uint8_t registerUserTimer(fp_utimer fp, uint16_t offset, void *state);

//-------------------------------------------------------------------------------------------------------
//  void deleteUserTimer(uint16_t id)
//
//  DESCRIPTION:
//		Deletes and deactivates an active usertimer
//
//---------------------------------------------------------------------------------------------------
extern void deleteUserTimer(uint8_t id);

#endif /*USERTIMER_H_*/

