package carrera;

import carrera.data.CarreraGame;
import carrera.data.CarreraGameEvent;
import carrera.data.CarreraGameListener;
import carrera.data.CarreraGames;
import carrera.data.PlotDataListener;
import carrera.dialog.NewGameDialog;
import carrera.dialog.OptionDialog;
import carrera.dialog.PlotView;
import carrera.dialog.SerialConnectionDialog;
import carrera.serial.SerialConnectionDataEvent;
import carrera.serial.SerialConnectionDataListener;
import carrera.serial.SerialConnectionDefaults;
import carrera.serial.SerialConnectionEvent;
import carrera.serial.SerialConnectionEventListener;
import carrera.serial.SerialConnectionManager;
import java.awt.Color;
import java.awt.Component;

/**
 *
 * @author  hillebra
 */
public class MainView extends javax.swing.JFrame implements
        SerialConnectionEventListener, SerialConnectionDataListener, CarreraGameListener {

    private Main main;
    
    /** Die ID der seriellen Verbindung */
    private int connectionID = -1;
    
    /** Alternativer Listener für serielle Daten des Plot */
    private PlotDataListener plotDataListener;
    
    /** Creates new form MainView */
    public MainView(Main main) {
        this.main = main;
        this.plotDataListener = new PlotDataListener(main);
        initComponents();
        main.addGameListener(this);
        SerialConnectionManager.addSerialConnectionEventListener(this);
        mainPanel.add(new GameCanvas(main));
    }
    
    /**
     * Zeigt ein Fenster mit einer Fehlermeldung an.
     *
     * @param title Der Fenstertitel.
     * @param msg Die Nachricht, die angezeigt werden soll.
     */
    public void showErrorMessage(final String title, final String msg) {
        final MainView parent = this;
        
        if (java.awt.EventQueue.isDispatchThread()) {
            javax.swing.JOptionPane.showMessageDialog(parent, msg, title,
                    javax.swing.JOptionPane.ERROR_MESSAGE);
        } else {
            javax.swing.SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    javax.swing.JOptionPane.showMessageDialog(parent, msg, title,
                        javax.swing.JOptionPane.ERROR_MESSAGE);
                }
            });
        }
    }
    
    /**
     * Zeigt ein Fenster mit einer Infomeldung an.
     *
     * @param title Der Fenstertitel.
     * @param msg Die Nachricht, die angezeigt werden soll.
     */
    public void showInfoMessage(final String title, final String msg) {
        final MainView parent = this;
        
        if (java.awt.EventQueue.isDispatchThread()) {
            javax.swing.JOptionPane.showMessageDialog(parent, msg, title,
                    javax.swing.JOptionPane.INFORMATION_MESSAGE);
        } else {
            javax.swing.SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    javax.swing.JOptionPane.showMessageDialog(parent, msg, title,
                        javax.swing.JOptionPane.INFORMATION_MESSAGE);
                }
            });
        }
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        btnGrpGameCtrl = new javax.swing.ButtonGroup();
        plotPopup = new javax.swing.JPopupMenu();
        miShow = new javax.swing.JMenuItem();
        miPlotSep = new javax.swing.JSeparator();
        miOwnSerial = new javax.swing.JMenuItem();
        miOwnSerialClose = new javax.swing.JMenuItem();
        mainPanel = new javax.swing.JPanel();
        pTool = new javax.swing.JPanel();
        toolBar = new javax.swing.JToolBar();
        bNewGame = new javax.swing.JButton();
        sep3 = new javax.swing.JToolBar.Separator();
        bStart = new javax.swing.JButton();
        bStop = new javax.swing.JButton();
        sep4 = new javax.swing.JToolBar.Separator();
        bHighScore = new javax.swing.JButton();
        bDeleteHighScore = new javax.swing.JButton();
        bPlot = new javax.swing.JButton();
        menuBar = new javax.swing.JMenuBar();
        mFile = new javax.swing.JMenu();
        miSerial = new javax.swing.JMenuItem();
        miSerialClose = new javax.swing.JMenuItem();
        sp2 = new javax.swing.JSeparator();
        miNewGame = new javax.swing.JMenuItem();
        sp1 = new javax.swing.JSeparator();
        miProperties = new javax.swing.JMenuItem();
        sp3 = new javax.swing.JSeparator();
        miExit = new javax.swing.JMenuItem();
        mAbout = new javax.swing.JMenu();
        miAbout = new javax.swing.JMenuItem();

        plotPopup.addPopupMenuListener(new javax.swing.event.PopupMenuListener() {
            public void popupMenuCanceled(javax.swing.event.PopupMenuEvent evt) {
            }
            public void popupMenuWillBecomeInvisible(javax.swing.event.PopupMenuEvent evt) {
            }
            public void popupMenuWillBecomeVisible(javax.swing.event.PopupMenuEvent evt) {
                plotPopupPopupMenuWillBecomeVisible(evt);
            }
        });

        miShow.setText("Anzeigen");
        miShow.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                miShowActionPerformed(evt);
            }
        });
        plotPopup.add(miShow);
        plotPopup.add(miPlotSep);

        miOwnSerial.setText("Neue serielle Verbindung...");
        miOwnSerial.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                miOwnSerialActionPerformed(evt);
            }
        });
        plotPopup.add(miOwnSerial);

        miOwnSerialClose.setText("Serielle Verbindung trennen");
        miOwnSerialClose.setEnabled(false);
        miOwnSerialClose.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                miOwnSerialCloseActionPerformed(evt);
            }
        });
        plotPopup.add(miOwnSerialClose);

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle(Main.PROGRAM_NAME);
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                formWindowClosing(evt);
            }
        });

        mainPanel.setLayout(new java.awt.BorderLayout());
        getContentPane().add(mainPanel, java.awt.BorderLayout.CENTER);

        pTool.setBorder(javax.swing.BorderFactory.createCompoundBorder(
            javax.swing.BorderFactory.createMatteBorder(0, 0, 1, 0, Color.WHITE),
            javax.swing.BorderFactory.createMatteBorder(0, 0, 1, 0, Color.GRAY)));
    pTool.setLayout(new java.awt.BorderLayout());

    toolBar.setFloatable(false);
    toolBar.setRollover(true);

    bNewGame.setIcon(new javax.swing.ImageIcon(getClass().getResource("/carrera/img/controller_add.png"))); // NOI18N
    bNewGame.setText("Neu");
    bNewGame.setToolTipText("Neues Spiel");
    bNewGame.setEnabled(false);
    bNewGame.setFocusable(false);
    bNewGame.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
    bNewGame.setMaximumSize(new java.awt.Dimension(50, 41));
    bNewGame.setMinimumSize(new java.awt.Dimension(50, 41));
    bNewGame.setPreferredSize(new java.awt.Dimension(50, 41));
    bNewGame.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
    bNewGame.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(java.awt.event.ActionEvent evt) {
            bNewGameActionPerformed(evt);
        }
    });
    toolBar.add(bNewGame);
    toolBar.add(sep3);

    bStart.setIcon(new javax.swing.ImageIcon(getClass().getResource("/carrera/img/control_play_blue.png"))); // NOI18N
    bStart.setText("Start");
    bStart.setToolTipText("Rennen starten");
    btnGrpGameCtrl.add(bStart);
    bStart.setEnabled(false);
    bStart.setFocusable(false);
    bStart.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
    bStart.setMaximumSize(new java.awt.Dimension(50, 41));
    bStart.setMinimumSize(new java.awt.Dimension(50, 41));
    bStart.setPreferredSize(new java.awt.Dimension(50, 41));
    bStart.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
    bStart.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(java.awt.event.ActionEvent evt) {
            bStartActionPerformed(evt);
        }
    });
    toolBar.add(bStart);

    bStop.setIcon(new javax.swing.ImageIcon(getClass().getResource("/carrera/img/control_stop_blue.png"))); // NOI18N
    bStop.setText("Stopp");
    bStop.setToolTipText("Rennen stoppen");
    btnGrpGameCtrl.add(bStop);
    bStop.setEnabled(false);
    bStop.setFocusable(false);
    bStop.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
    bStop.setMaximumSize(new java.awt.Dimension(50, 41));
    bStop.setMinimumSize(new java.awt.Dimension(50, 41));
    bStop.setPreferredSize(new java.awt.Dimension(50, 41));
    bStop.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
    bStop.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(java.awt.event.ActionEvent evt) {
            bStopActionPerformed(evt);
        }
    });
    toolBar.add(bStop);
    toolBar.add(sep4);

    bHighScore.setIcon(new javax.swing.ImageIcon(getClass().getResource("/carrera/img/rosette.png"))); // NOI18N
    bHighScore.setText("Top 10");
    bHighScore.setToolTipText("Anzeigen der Top 10");
    bHighScore.setFocusable(false);
    bHighScore.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
    bHighScore.setMaximumSize(new java.awt.Dimension(50, 41));
    bHighScore.setMinimumSize(new java.awt.Dimension(50, 41));
    bHighScore.setPreferredSize(new java.awt.Dimension(50, 41));
    bHighScore.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
    bHighScore.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(java.awt.event.ActionEvent evt) {
            bHighScoreActionPerformed(evt);
        }
    });
    toolBar.add(bHighScore);

    bDeleteHighScore.setIcon(new javax.swing.ImageIcon(getClass().getResource("/carrera/img/cross.png"))); // NOI18N
    bDeleteHighScore.setText("Top 10");
    bDeleteHighScore.setToolTipText("Leeren der Top 10");
    bDeleteHighScore.setFocusable(false);
    bDeleteHighScore.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
    bDeleteHighScore.setMaximumSize(new java.awt.Dimension(50, 41));
    bDeleteHighScore.setMinimumSize(new java.awt.Dimension(50, 41));
    bDeleteHighScore.setPreferredSize(new java.awt.Dimension(50, 41));
    bDeleteHighScore.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
    bDeleteHighScore.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(java.awt.event.ActionEvent evt) {
            bDeleteHighScoreActionPerformed(evt);
        }
    });
    toolBar.add(bDeleteHighScore);

    bPlot.setIcon(new javax.swing.ImageIcon(getClass().getResource("/carrera/img/chart_curve.png"))); // NOI18N
    bPlot.setText("Plot");
    bPlot.setToolTipText("Anzeige der Beschleunigungswerte");
    bPlot.setFocusable(false);
    bPlot.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
    bPlot.setMaximumSize(new java.awt.Dimension(50, 41));
    bPlot.setMinimumSize(new java.awt.Dimension(50, 41));
    bPlot.setPreferredSize(new java.awt.Dimension(50, 41));
    bPlot.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
    bPlot.addMouseListener(new java.awt.event.MouseAdapter() {
        public void mouseClicked(java.awt.event.MouseEvent evt) {
            bPlotMouseClicked(evt);
        }
    });
    toolBar.add(bPlot);

    pTool.add(toolBar, java.awt.BorderLayout.NORTH);

    getContentPane().add(pTool, java.awt.BorderLayout.NORTH);

    mFile.setMnemonic('D');
    mFile.setText("Datei");

    miSerial.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_S, java.awt.event.InputEvent.CTRL_MASK));
    miSerial.setIcon(new javax.swing.ImageIcon(getClass().getResource("/carrera/img/connect.png"))); // NOI18N
    miSerial.setMnemonic('S');
    miSerial.setText("Serielle Schnittstelle...");
    miSerial.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(java.awt.event.ActionEvent evt) {
            miSerialActionPerformed(evt);
        }
    });
    mFile.add(miSerial);

    miSerialClose.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_T, java.awt.event.InputEvent.CTRL_MASK));
    miSerialClose.setIcon(new javax.swing.ImageIcon(getClass().getResource("/carrera/img/disconnect.png"))); // NOI18N
    miSerialClose.setMnemonic('t');
    miSerialClose.setText("Serielle Verbindung trennen");
    miSerialClose.setEnabled(false);
    miSerialClose.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(java.awt.event.ActionEvent evt) {
            miSerialCloseActionPerformed(evt);
        }
    });
    mFile.add(miSerialClose);
    mFile.add(sp2);

    miNewGame.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_N, java.awt.event.InputEvent.CTRL_MASK));
    miNewGame.setIcon(new javax.swing.ImageIcon(getClass().getResource("/carrera/img/controller_add.png"))); // NOI18N
    miNewGame.setMnemonic('N');
    miNewGame.setText("Neues Spiel...");
    miNewGame.setEnabled(false);
    miNewGame.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(java.awt.event.ActionEvent evt) {
            miNewGameActionPerformed(evt);
        }
    });
    mFile.add(miNewGame);
    mFile.add(sp1);

    miProperties.setText("Einstellungen...");
    miProperties.setEnabled(false);
    miProperties.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(java.awt.event.ActionEvent evt) {
            miPropertiesActionPerformed(evt);
        }
    });
    mFile.add(miProperties);
    mFile.add(sp3);

    miExit.setMnemonic('B');
    miExit.setText("Beenden");
    miExit.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(java.awt.event.ActionEvent evt) {
            miExitActionPerformed(evt);
        }
    });
    mFile.add(miExit);

    menuBar.add(mFile);

    mAbout.setMnemonic('?');
    mAbout.setText("?");

    miAbout.setMnemonic('I');
    miAbout.setText("Info");
    miAbout.addActionListener(new java.awt.event.ActionListener() {
        public void actionPerformed(java.awt.event.ActionEvent evt) {
            miAboutActionPerformed(evt);
        }
    });
    mAbout.add(miAbout);

    menuBar.add(mAbout);

    setJMenuBar(menuBar);

    java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
    setBounds((screenSize.width-579)/2, (screenSize.height-489)/2, 579, 489);
    }// </editor-fold>//GEN-END:initComponents

private void miExitActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_miExitActionPerformed
    main.exit();
}//GEN-LAST:event_miExitActionPerformed

private void miSerialActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_miSerialActionPerformed
    SerialConnectionManager.openConnectionDialog(this);
}//GEN-LAST:event_miSerialActionPerformed

private void miSerialCloseActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_miSerialCloseActionPerformed
    SerialConnectionManager.closeConnection(connectionID);
}//GEN-LAST:event_miSerialCloseActionPerformed

private void miNewGameActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_miNewGameActionPerformed
    newGameAction();
}//GEN-LAST:event_miNewGameActionPerformed

private void bStartActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bStartActionPerformed
    main.startGame();
}//GEN-LAST:event_bStartActionPerformed

private void bStopActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bStopActionPerformed
    main.endGame(false);
}//GEN-LAST:event_bStopActionPerformed

private void formWindowClosing(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosing
    main.exit();
}//GEN-LAST:event_formWindowClosing

private void miAboutActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_miAboutActionPerformed
    showInfoMessage("Info zu " + Main.PROGRAM_NAME, "<html><b>" + Main.PROGRAM_NAME
            + " " + Main.PROGRAM_VERSION + "</b>\n\n" + Main.PROGRAM_COPYRIGHT);
}//GEN-LAST:event_miAboutActionPerformed

private void bNewGameActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bNewGameActionPerformed
    newGameAction();
}//GEN-LAST:event_bNewGameActionPerformed

private void bHighScoreActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bHighScoreActionPerformed
    main.setGameMode(CarreraGames.MODE_DISPLAY_HIGH_SCORE);
}//GEN-LAST:event_bHighScoreActionPerformed

private void bPlotMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_bPlotMouseClicked
    Component c = evt.getComponent();
    plotPopup.show(c, 0, c.getHeight());
}//GEN-LAST:event_bPlotMouseClicked

private void miShowActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_miShowActionPerformed
    main.show(new PlotView(this, false, main.getPlotData()));
}//GEN-LAST:event_miShowActionPerformed

private void miOwnSerialActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_miOwnSerialActionPerformed
    SerialConnectionManager.openConnectionDialog(plotDataListener);
}//GEN-LAST:event_miOwnSerialActionPerformed

private void plotPopupPopupMenuWillBecomeVisible(javax.swing.event.PopupMenuEvent evt) {//GEN-FIRST:event_plotPopupPopupMenuWillBecomeVisible
    miOwnSerial.setEnabled(plotDataListener.getSerialConnectionID() == -1);
    miOwnSerialClose.setEnabled(plotDataListener.getSerialConnectionID() != -1);
}//GEN-LAST:event_plotPopupPopupMenuWillBecomeVisible

private void miOwnSerialCloseActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_miOwnSerialCloseActionPerformed
    SerialConnectionManager.closeConnection(plotDataListener.getSerialConnectionID());
}//GEN-LAST:event_miOwnSerialCloseActionPerformed

private void miPropertiesActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_miPropertiesActionPerformed
    main.show(new OptionDialog(this, true, main));
}//GEN-LAST:event_miPropertiesActionPerformed

private void bDeleteHighScoreActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bDeleteHighScoreActionPerformed
    int option = javax.swing.JOptionPane.showConfirmDialog(this,
                    "Soll die aktuelle Highscore geleert werden?",
                    "Highscore leeren?",
                    javax.swing.JOptionPane.YES_NO_OPTION);
    if (option == javax.swing.JOptionPane.NO_OPTION) return;
    main.getCarreraGames().deleteHighScore();
    main.notifyGameListeners(new CarreraGameEvent(this, CarreraGameEvent.EVENT_HIGHSCORE_DELETE));
}//GEN-LAST:event_bDeleteHighScoreActionPerformed

    private void newGameAction() {
        if (main.getCurrentGame() != null && !main.getCurrentGame().isOver()) {
            int option = javax.swing.JOptionPane.showConfirmDialog(this,
                    "Aktuelles Spiel abbrechen?", "Spiel beenden?",
                    javax.swing.JOptionPane.YES_NO_OPTION);
            if (option == javax.swing.JOptionPane.NO_OPTION) return;
            main.closeCurrentGame();
        }
        main.show(new NewGameDialog(this, true, main));
    }

    public void carreraGameChanged(CarreraGameEvent event) {
        switch (event.getType()) {
            case CarreraGameEvent.EVENT_NEW_GAME:
                bStart.setEnabled(true);
                bHighScore.setEnabled(false);
                bDeleteHighScore.setEnabled(false);
                miProperties.setEnabled(true);
                break;
            case CarreraGameEvent.EVENT_GAME_START:
                bStart.setEnabled(false);
                bStop.setEnabled(true);
                miProperties.setEnabled(false);
                break;
            case CarreraGameEvent.EVENT_GAME_END:
                bStart.setEnabled(false);
                bStop.setEnabled(false);
                bHighScore.setEnabled(true);
                bDeleteHighScore.setEnabled(true);
                miProperties.setEnabled(true);
        }
    }

    public void serialConnectionEvent(SerialConnectionEvent event) {
        switch (event.getType()) {
            case SerialConnectionEvent.SERIAL_CONNECTION_CREATE_REQUEST:
                SerialConnectionDialog d = new SerialConnectionDialog(MainView.this,
                        true, (SerialConnectionDataListener) event.getSource());
                main.show(d);
                break;
        }
    }

    public SerialConnectionDefaults getSerialConnectionDefaults() {
        return new SerialConnectionDefaults(115200, 8, 0, 0);
    }

    public int getSerialConnectionID() {
        return connectionID;
    }

    public void serialConnectionEstablished(int connectionID) {
        this.connectionID = connectionID;
        miSerialClose.setEnabled(true);
        miSerial.setEnabled(false);
        miNewGame.setEnabled(true);
        bNewGame.setEnabled(true);
        miProperties.setEnabled(true);
    }

    public void serialConnectionClosing() {
        main.closeCurrentGame();
    }
    
    public void serialConnectionClosed() {
        this.connectionID = -1;
        miSerialClose.setEnabled(false);
        miSerial.setEnabled(true);
        miNewGame.setEnabled(false);
        bNewGame.setEnabled(false);
        miProperties.setEnabled(false);
    }
    
    public void serialConnectionDataEvent(SerialConnectionDataEvent event) {
        String str = event.getData();
        
        if (str.equalsIgnoreCase("A")) {
            main.notifyNewRound(CarreraGame.DRIVER_CAR);
            return;
        } else if(str.equalsIgnoreCase("H")) {
            main.notifyNewRound(CarreraGame.DRIVER_HUMAN);
            return;
        } else if(str.equalsIgnoreCase("B")) {
            main.setVehicleStatus(0);
            return;
        } else if(str.equalsIgnoreCase("T")) {
            main.setVehicleStatus(1);
            return;
        } else {
            System.out.println("GOT: " + str);
        }
        
        // Nur benutzen, falls keine eigene serielle Verbindung
        if (plotDataListener.getSerialConnectionID() == -1) {
            main.parseAndAddXYData(str);
        }
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton bDeleteHighScore;
    private javax.swing.JButton bHighScore;
    private javax.swing.JButton bNewGame;
    private javax.swing.JButton bPlot;
    private javax.swing.JButton bStart;
    private javax.swing.JButton bStop;
    private javax.swing.ButtonGroup btnGrpGameCtrl;
    private javax.swing.JMenu mAbout;
    private javax.swing.JMenu mFile;
    private javax.swing.JPanel mainPanel;
    private javax.swing.JMenuBar menuBar;
    private javax.swing.JMenuItem miAbout;
    private javax.swing.JMenuItem miExit;
    private javax.swing.JMenuItem miNewGame;
    private javax.swing.JMenuItem miOwnSerial;
    private javax.swing.JMenuItem miOwnSerialClose;
    private javax.swing.JSeparator miPlotSep;
    private javax.swing.JMenuItem miProperties;
    private javax.swing.JMenuItem miSerial;
    private javax.swing.JMenuItem miSerialClose;
    private javax.swing.JMenuItem miShow;
    private javax.swing.JPanel pTool;
    private javax.swing.JPopupMenu plotPopup;
    private javax.swing.JToolBar.Separator sep3;
    private javax.swing.JToolBar.Separator sep4;
    private javax.swing.JSeparator sp1;
    private javax.swing.JSeparator sp2;
    private javax.swing.JSeparator sp3;
    private javax.swing.JToolBar toolBar;
    // End of variables declaration//GEN-END:variables

}
