package carrera.data;

import carrera.Main;
import java.util.Iterator;
import java.util.TreeSet;

/**
 *
 * @author hillebra
 */
public class CarreraGames implements java.io.Serializable {

    private int gameCount;
    private CarreraPlayer highScoreAuto;
    private TreeSet<CarreraPlayer> highScore;
    
    public static final int MODE_DISPLAY_BOOTSCREEN = 0;
    public static final int MODE_DISPLAY_CURRENT_GAME = 1;
    public static final int MODE_DISPLAY_HIGH_SCORE = 2;
    
    public CarreraGames() {
        gameCount = 0;
        highScore = new TreeSet<CarreraPlayer>();
    }
    
    public int getGameCount() {
        return gameCount;
    }
    
    public int getHighScoreCount() {
        return highScore.size();
    }
    
    public Iterator<CarreraPlayer> getHighScore() {
        return highScore.iterator();
    }
    
    public void deleteHighScore() {
        gameCount = 0;
        highScore.clear();
        highScoreAuto = null;
    }
    
    public int getHighScorePosition(CarreraPlayer player) {
        int position = 0;
        boolean found = false;
        Iterator<CarreraPlayer> iter = getHighScore();
        while (iter.hasNext()) {
            position++;
            CarreraPlayer tmp = iter.next();
            if (player.isAuto() && tmp.isAuto()) { found = true; break; }
            if (tmp == player) { found = true; break; }
        }
        return found ? position : -1;
    }
    
    public CarreraGame createGame(Main main, String autoName, String manualName) {
        CarreraGame game = new CarreraGame(main, autoName, manualName);
        gameCount++;
        return game;
    }
    
    public void storeGame(CarreraGame game) {
        if (game.wasSuccess()) {
            CarreraPlayer p = game.getAutomaticPlayer();
            if (p.didFinish() && p.isAuto()) {
                if (highScoreAuto != null) {
                    if (highScoreAuto.getOverallTime() > p.getOverallTime()) {
                        highScore.remove(highScoreAuto);
                        highScoreAuto = p;
                        highScore.add(p);
                    }
                } else {
                    highScoreAuto = p;
                    highScore.add(p);
                }  
            }
            if (game.getManualPlayer().didFinish()) {
                highScore.add(game.getManualPlayer());
            }
        }
    }
 
}
