package carrera.dialog;

import carrera.data.plot.PlotData;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JCheckBox;
import javax.swing.JPanel;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.labels.XYSeriesLabelGenerator;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.data.xy.XYDataset;

/**
 *
 * @author  hillebra
 */
public class PlotView extends javax.swing.JDialog {

    private PlotData plotData;

    /** Creates new form PlotView */
    public PlotView(java.awt.Frame parent, boolean modal, PlotData plotData) {
        super(parent, modal);
        initComponents();
        this.plotData = plotData;
        initChart();
    }
    
    private void addChart(final JFreeChart chart, String[] labels) {
        JPanel container = new JPanel();
        container.setLayout(new java.awt.BorderLayout());
        JPanel pCheckBox = new JPanel();
        pCheckBox.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.CENTER));
        
        for (int i = 0; i < labels.length; i++) {
            JCheckBox cb = new JCheckBox(labels[i], true);
            cb.setActionCommand("" + i);
            cb.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    JCheckBox source = (JCheckBox) e.getSource();
                    String action = e.getActionCommand();
                    chart.getXYPlot().getRenderer().setSeriesVisible(
                            Integer.parseInt(action), source.isSelected());
                }
            });
            pCheckBox.add(cb);
        }
        
        ChartPanel cp = new ChartPanel(chart);
        cp.setPreferredSize(new Dimension(500, 250));
        container.add(cp, java.awt.BorderLayout.CENTER);
        container.add(pCheckBox, java.awt.BorderLayout.SOUTH);
        mainPanel.add(container);
    }
    
    private void initChart() {
        JFreeChart chartAcc = ChartFactory.createXYLineChart("Beschleunigungssensor",
                "Samples", "ADC-Wert", plotData.getAccDataset(),
                PlotOrientation.VERTICAL, true, false, false);
        chartAcc.setBackgroundPaint(mainPanel.getBackground());
        chartAcc.getXYPlot().getRangeAxis().setRange(-1000, 1000);
        chartAcc.getXYPlot().getRangeAxis().setAutoRange(false);
        //chartAcc.getXYPlot().getRenderer().setSeriesPaint(1, Color.LIGHT_GRAY);
        final String[] accLabels = new String[] {"X", "X (RAW)",
                "Y", "Y (RAW)", "Z", "Z (RAW)"};
        chartAcc.getXYPlot().getRenderer().setLegendItemLabelGenerator(new XYSeriesLabelGenerator() {
            public String generateLabel(XYDataset dataset, int series) {
                return accLabels[series];
            }
        });
        addChart(chartAcc, accLabels);

        /*
        JFreeChart chartGyro = ChartFactory.createXYLineChart("Gyroskop",
                "Samples", "ADC-Wert", plotData.getGyroDataset(),
                PlotOrientation.VERTICAL, true, false, false);
        chartGyro.setBackgroundPaint(mainPanel.getBackground());
        chartGyro.getXYPlot().getRangeAxis().setRange(-1000, 1000);
        chartGyro.getXYPlot().getRangeAxis().setAutoRange(false);
        //chartGyro.getXYPlot().getRenderer().setSeriesPaint(1, Color.LIGHT_GRAY);
        final String[] gyroLabels = new String[] {"X", "X (RAW)",
                "Y", "Y (RAW)", "Z", "Z (RAW)"};
        chartGyro.getXYPlot().getRenderer().setLegendItemLabelGenerator(new XYSeriesLabelGenerator() {
            public String generateLabel(XYDataset dataset, int series) {
                return gyroLabels[series];
            }
        });
        addChart(chartGyro, gyroLabels);
        */
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        scrollPane = new javax.swing.JScrollPane();
        mainPanel = new javax.swing.JPanel();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Carrera: Sensorik");

        mainPanel.setLayout(new java.awt.GridLayout(1, 1));
        scrollPane.setViewportView(mainPanel);

        getContentPane().add(scrollPane, java.awt.BorderLayout.CENTER);

        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width-605)/2, (screenSize.height-593)/2, 605, 593);
    }// </editor-fold>//GEN-END:initComponents

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel mainPanel;
    private javax.swing.JScrollPane scrollPane;
    // End of variables declaration//GEN-END:variables

}
