package carrera.dialog;

import carrera.serial.SerialConnectionDataListener;
import carrera.serial.SerialConnectionDefaults;
import carrera.serial.SerialConnectionException;
import carrera.serial.SerialConnectionManager;
import carrera.serial.SerialConnectionProperties;
import java.util.Enumeration;
import javax.swing.DefaultComboBoxModel;
import gnu.io.SerialPort;
import gnu.io.CommPortIdentifier;

/**
 * Einstellungsdialog für serielle Verbindungen.
 * 
 * @version 1.0, 2008-01-14
 * @author  Thomas Hillebrandt
 * @since   1.0
 */
public class SerialConnectionDialog extends javax.swing.JDialog {
    
    private SerialConnectionDataListener connectionRequestSource;
    
    /** Creates new form SerialConnectionDialog */
    public SerialConnectionDialog(java.awt.Frame parent, boolean modal,
            SerialConnectionDataListener owner) {
        super(parent, modal);
        if (owner == null) {
            throw new IllegalArgumentException();
        }
        connectionRequestSource = owner;
        initComponents();
        initDialog();
        getRootPane().setDefaultButton(bOk);
    }
    
    private void initDialog() {
        DefaultComboBoxModel dcm = new DefaultComboBoxModel();
        Enumeration ports = CommPortIdentifier.getPortIdentifiers();
        while (ports.hasMoreElements()) {
            CommPortIdentifier cpi = (CommPortIdentifier) ports.nextElement();
            if (cpi.getPortType() == CommPortIdentifier.PORT_SERIAL) {
                dcm.addElement(cpi.getName());
            }
        }
        cbPorts.setModel(dcm);
        
        SerialConnectionDefaults dflts =
                connectionRequestSource.getSerialConnectionDefaults();
        
        tfSpeed.setText(new Integer(dflts.getBaudrate()).toString());
        
        dcm = new DefaultComboBoxModel();
        dcm.addElement(SerialPort.DATABITS_5);
        dcm.addElement(SerialPort.DATABITS_6);
        dcm.addElement(SerialPort.DATABITS_7);
        dcm.addElement(SerialPort.DATABITS_8);
        cbDataBits.setModel(dcm);
        cbDataBits.setSelectedItem(dflts.getDatabits());
        
        dcm = new DefaultComboBoxModel();
        dcm.addElement(1);
        dcm.addElement(2);
        dcm.addElement(1.5);
        cbStopBits.setModel(dcm);
        cbStopBits.setSelectedIndex(dflts.getStopbits());
        
        dcm = new DefaultComboBoxModel();
        dcm.addElement("None");
        dcm.addElement("Odd");
        dcm.addElement("Even");
        dcm.addElement("Mark");
        dcm.addElement("Space");
        cbParity.setModel(dcm);
        cbParity.setSelectedIndex(dflts.getParity());
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel1 = new javax.swing.JPanel();
        cbPorts = new javax.swing.JComboBox();
        lPort = new javax.swing.JLabel();
        lSpeed = new javax.swing.JLabel();
        tfSpeed = new javax.swing.JTextField();
        cbDataBits = new javax.swing.JComboBox();
        cbStopBits = new javax.swing.JComboBox();
        cbParity = new javax.swing.JComboBox();
        lDataBits = new javax.swing.JLabel();
        lStopBits = new javax.swing.JLabel();
        lParity = new javax.swing.JLabel();
        bCancel = new javax.swing.JButton();
        bOk = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Serielle Verbindung");
        setModal(true);
        setName("serialConnectionDialog"); // NOI18N
        setResizable(false);
        getContentPane().setLayout(null);

        jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder("Einstellungen"));
        jPanel1.setLayout(null);
        jPanel1.add(cbPorts);
        cbPorts.setBounds(140, 30, 90, 20);

        lPort.setText("Port:");
        jPanel1.add(lPort);
        lPort.setBounds(20, 30, 70, 14);

        lSpeed.setText("Baudrate:");
        jPanel1.add(lSpeed);
        lSpeed.setBounds(20, 70, 110, 14);

        tfSpeed.setText("115200");
        jPanel1.add(tfSpeed);
        tfSpeed.setBounds(140, 70, 90, 20);
        jPanel1.add(cbDataBits);
        cbDataBits.setBounds(140, 110, 90, 20);
        jPanel1.add(cbStopBits);
        cbStopBits.setBounds(140, 150, 90, 20);
        jPanel1.add(cbParity);
        cbParity.setBounds(140, 190, 90, 20);

        lDataBits.setText("Datenbits:");
        jPanel1.add(lDataBits);
        lDataBits.setBounds(20, 110, 110, 14);

        lStopBits.setText("Stoppbits:");
        jPanel1.add(lStopBits);
        lStopBits.setBounds(20, 150, 110, 14);

        lParity.setText("Paritaet:");
        jPanel1.add(lParity);
        lParity.setBounds(20, 190, 110, 14);

        getContentPane().add(jPanel1);
        jPanel1.setBounds(10, 10, 260, 230);

        bCancel.setText("Abbrechen");
        bCancel.setMargin(new java.awt.Insets(2, 5, 2, 5));
        bCancel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bCancelActionPerformed(evt);
            }
        });
        getContentPane().add(bCancel);
        bCancel.setBounds(150, 250, 80, 23);

        bOk.setText("OK");
        bOk.setMargin(new java.awt.Insets(2, 5, 2, 5));
        bOk.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bOkActionPerformed(evt);
            }
        });
        getContentPane().add(bOk);
        bOk.setBounds(50, 250, 80, 23);

        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width-285)/2, (screenSize.height-313)/2, 285, 313);
    }// </editor-fold>//GEN-END:initComponents

    private void bCancelActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bCancelActionPerformed
        dispose();
    }//GEN-LAST:event_bCancelActionPerformed

    private void bOkActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bOkActionPerformed
        if (cbPorts.getModel().getSize() == 0) {
            javax.swing.JOptionPane.showMessageDialog(getParent(),
                    "Kein Port vorhanden!", "Fehler",
                    javax.swing.JOptionPane.ERROR_MESSAGE);
            return;
        }
        
        try {
            SerialConnectionProperties scp = new SerialConnectionProperties(
                    cbPorts.getSelectedItem().toString(),
                    Integer.parseInt(tfSpeed.getText()),
                    (Integer) cbDataBits.getSelectedItem(),
                    cbStopBits.getSelectedIndex() + 1,
                    cbParity.getSelectedIndex());
            SerialConnectionManager.openConnection(connectionRequestSource, scp);
            dispose();
        } catch (NumberFormatException e) {
            javax.swing.JOptionPane.showMessageDialog(getParent(),
                    "Bitte überprüfen Sie die Angabe der Geschwindigkeit!",
                    "Fehler", javax.swing.JOptionPane.ERROR_MESSAGE);
        } catch (SerialConnectionException e) {
            String errorMsg = null;
            switch (e.getType()) {
                case SerialConnectionException.NO_SUCH_PORT:
                    errorMsg = "Der Port ist nicht vorhanden!";
                    break;
                case SerialConnectionException.PORT_IN_USE:
                    errorMsg = "Der Port wird bereits benutzt!";
                    break;
                case SerialConnectionException.IO_ERROR:
                    errorMsg = "IO-Fehler!";
                    break;
                case SerialConnectionException.UNSUPPORTED_COMM_OPERATION:
                    errorMsg = "Operation wird nicht unterstützt!";
                    break;
                case SerialConnectionException.NO_DATA_LISTENER:
                    errorMsg = "Kein Datenempfänger für serielle Daten vorhanden!";
                    break;
            }
            javax.swing.JOptionPane.showMessageDialog(getParent(), errorMsg,
                    "Fehler", javax.swing.JOptionPane.ERROR_MESSAGE);
        }
    }//GEN-LAST:event_bOkActionPerformed
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton bCancel;
    private javax.swing.JButton bOk;
    private javax.swing.JComboBox cbDataBits;
    private javax.swing.JComboBox cbParity;
    private javax.swing.JComboBox cbPorts;
    private javax.swing.JComboBox cbStopBits;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JLabel lDataBits;
    private javax.swing.JLabel lParity;
    private javax.swing.JLabel lPort;
    private javax.swing.JLabel lSpeed;
    private javax.swing.JLabel lStopBits;
    private javax.swing.JTextField tfSpeed;
    // End of variables declaration//GEN-END:variables
    
}
